from impresso.api_client.client import AuthenticatedClient
from impresso.resources.content_items import ContentItemsResource
from impresso.resources.collections import CollectionsResource
from impresso.resources.entities import EntitiesResource
from impresso.resources.newspapers import NewspapersResource
from impresso.resources.search import SearchResource
from impresso.resources.text_reuse import TextReuseDomain
from impresso.resources.tools import ToolsResource


class ImpressoApiResourcesBase:
    """Base class for the Impresso client that contains resources."""

    def __init__(self, api_client: AuthenticatedClient) -> None:
        self._api_client = api_client

    @property
    def search(self) -> SearchResource:
        return SearchResource(self._api_client)

    @property
    def content_items(self) -> ContentItemsResource:
        return ContentItemsResource(self._api_client)

    @property
    def text_reuse(self) -> TextReuseDomain:
        return TextReuseDomain(self._api_client)

    @property
    def newspapers(self) -> NewspapersResource:
        return NewspapersResource(self._api_client)

    @property
    def collections(self) -> CollectionsResource:
        return CollectionsResource(self._api_client)

    @property
    def entities(self) -> EntitiesResource:
        return EntitiesResource(self._api_client)

    @property
    def tools(self) -> ToolsResource:
        return ToolsResource(self._api_client)



from enum import Enum
from typing import Literal


class ContentItemLabelsItem(str, Enum):
    ARTICLE = "article"

    def __str__(self) -> str:
        return str(self.value)


ContentItemLabelsItemLiteral = Literal["article",]



from enum import Enum
from typing import Literal


class CollectableItemGroupContentType(str, Enum):
    A = "A"
    E = "E"
    I = "I"
    P = "P"

    def __str__(self) -> str:
        return str(self.value)


CollectableItemGroupContentTypeLiteral = Literal[
    "A",
    "E",
    "I",
    "P",
]



from enum import Enum
from typing import Literal


class ContentItemAccessRight(str, Enum):
    CLOSED = "Closed"
    NA = "na"
    OPENPRIVATE = "OpenPrivate"
    OPENPUBLIC = "OpenPublic"

    def __str__(self) -> str:
        return str(self.value)


ContentItemAccessRightLiteral = Literal[
    "Closed",
    "na",
    "OpenPrivate",
    "OpenPublic",
]



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="Entity")


@_attrs_define
class Entity:
    """An entity like location, person, etc

    Attributes:
        uid (str): Unique identifier of the entity
        relevance (int): Relevance of the entity in the document
    """

    uid: str
    relevance: int

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        relevance = self.relevance

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "relevance": relevance,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        relevance = d.pop("relevance")

        entity = cls(
            uid=uid,
            relevance=relevance,
        )

        return entity



from typing import Any
import matplotlib.pyplot as plt
import io
import base64

from pandas import DataFrame, json_normalize

from impresso.api_client.api.search import search
from impresso.api_client.api.search_facets import get_search_facet
from impresso.api_client.models.content_item_access_right import (
    ContentItemAccessRightLiteral,
)
from impresso.api_client.models.get_search_facet_id import (
    GetSearchFacetId,
    GetSearchFacetIdLiteral,
)
from impresso.api_client.models.get_search_facet_order_by import (
    GetSearchFacetOrderBy,
    GetSearchFacetOrderByLiteral,
)
from impresso.api_client.models.search_group_by import SearchGroupBy
from impresso.api_client.models.search_order_by import (
    SearchOrderBy,
    SearchOrderByLiteral,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_models import Article, BaseFind, Filter, Q, SearchFacet
from impresso.data_container import IT, DataContainer, T
from impresso.resources.base import DEFAULT_PAGE_SIZE, Resource
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal, get_enum_from_literal_required


class SearchResponseSchema(BaseFind):
    """Schema for the articles response."""

    data: list[Article]


class SearchDataContainer(DataContainer):
    """Response of a search call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()


class FacetDataContainer(DataContainer):
    """Response of a get facet call."""

    def __init__(
        self,
        data: IT,
        pydantic_model: type[T],
        limit: int | None,
        offset: int | None,
        web_app_search_result_url: str,
    ):
        super().__init__(data, pydantic_model, web_app_search_result_url)
        self._limit = limit
        self._offset = offset

    @property
    def raw(self) -> dict[str, Any]:
        """Return the data as a python dictionary."""
        return self._data.to_dict()

    @property
    def pydantic(self) -> SearchFacet:
        """Return the data as a pydantic model."""
        return self._pydantic_model.model_validate(self.raw)

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        if len(self.raw["buckets"]) == 0:
            return DataFrame()
        return json_normalize(self.raw["buckets"]).set_index("val")

    @property
    def size(self) -> int:
        """Current page size."""
        return len(self.raw.get("buckets", []))

    @property
    def total(self) -> int:
        """Total number of results."""
        return self.raw.get("numBuckets", 0)

    @property
    def limit(self) -> int:
        """Page size."""
        return self._limit or len(self.raw["buckets"])

    @property
    def offset(self) -> int:
        """Page offset."""
        return self._offset or 0

    def _get_preview_image_(self) -> str | None:
        if self.size == 0:
            return None
        return render_dataframe_chart_base64(
            self.df.index[:].values, self.df["count"].values
        )


class SearchResource(Resource):
    """Search articles in the impresso database."""

    name = "search"

    def find(
        self,
        q: str | AND[str] | OR[str] | None = None,
        order_by: SearchOrderByLiteral | None = None,
        limit: int | None = None,
        offset: int | None = None,
        with_text_contents: bool | None = False,
        title: str | AND[str] | OR[str] | None = None,
        front_page: bool | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        newspaper_id: str | OR[str] | None = None,
        date_range: DateRange | None = None,
        language: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        access_rights: (
            ContentItemAccessRightLiteral | OR[ContentItemAccessRightLiteral] | None
        ) = None,
        partner_id: str | OR[str] | None = None,
        text_reuse_cluster_id: str | OR[str] | None = None,
    ) -> SearchDataContainer:
        """
        Search for articles in Impresso.

        Args:
            q: Search term.
            order_by: Order by aspect.
            limit: Number of results to return.
            offset: Number of results to skip.

            with_text_contents: Return only articles with text contents.
            title: Return only articles that have this term or all/any of the terms in the title.
            front_page: Return only articles that were on the front page.
            entity_id: Return only articles that mention this entity or all/any of the entities.
            date_range: Return only articles that were published in this date range.
            language: Return only articles that are in this language or all/any of the languages.
            mention: Return only articles that mention an entity with this term or all/any of entities with the terms.
            topic_id: Return only articles that are about this topic or all/any of the topics.
            collection_id: Return only articles that are in this collection or all/any of the collections.
            country: Return only articles that are from this country or all/any of the countries.
            access_rights: Return only articles with this access right or all/any of the access rights.
            partner_id: Return only articles that are from this partner or all/any of the partners.
            text_reuse_cluster_id: Return only articles that are in this text reuse cluster or all/any of the clusters.

        Returns:
            _type_: _description_
        """

        filters = self._build_filters(
            string=q,
            with_text_contents=with_text_contents,
            title=title,
            front_page=front_page,
            entity_id=entity_id,
            newspaper_id=newspaper_id,
            date_range=date_range,
            language=language,
            mention=mention,
            topic_id=topic_id,
            collection_id=collection_id,
            country=country,
            access_rights=access_rights,
            partner_id=partner_id,
            text_reuse_cluster_id=text_reuse_cluster_id,
        )

        filters_pb = filters_as_protobuf(filters or [])

        result = search.sync(
            client=self._api_client,
            q=UNSET,
            order_by=(
                get_enum_from_literal(order_by, SearchOrderBy)
                if order_by is not None
                else UNSET
            ),
            group_by=get_enum_from_literal_required("articles", SearchGroupBy),
            filters=filters_pb if filters_pb else UNSET,
            limit=limit if limit is not None else DEFAULT_PAGE_SIZE,
            offset=offset if offset is not None else UNSET,
        )
        raise_for_error(result)
        return SearchDataContainer(
            result,
            SearchResponseSchema,
            _build_web_app_search_url(
                f"{self._get_web_app_base_url()}/search",
                order_by=order_by,
                filters=filters_pb,
                limit=limit,
                offset=offset,
            ),
        )

    def facet(
        self,
        facet: GetSearchFacetIdLiteral,
        q: str | AND[str] | OR[str] | None = None,
        order_by: GetSearchFacetOrderByLiteral | None = "value",
        limit: int | None = None,
        offset: int | None = None,
        with_text_contents: bool | None = False,
        title: str | AND[str] | OR[str] | None = None,
        front_page: bool | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        newspaper_id: str | OR[str] | None = None,
        date_range: DateRange | None = None,
        language: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        access_rights: (
            ContentItemAccessRightLiteral | OR[ContentItemAccessRightLiteral] | None
        ) = None,
        partner_id: str | OR[str] | None = None,
        text_reuse_cluster_id: str | OR[str] | None = None,
    ) -> FacetDataContainer:

        facet_id = get_enum_from_literal(facet, GetSearchFacetId)
        if isinstance(facet_id, Unset):
            raise ValueError(f"{facet} is not a valid value")

        filters = self._build_filters(
            string=q,
            with_text_contents=with_text_contents,
            title=title,
            front_page=front_page,
            entity_id=entity_id,
            newspaper_id=newspaper_id,
            date_range=date_range,
            language=language,
            mention=mention,
            topic_id=topic_id,
            collection_id=collection_id,
            country=country,
            access_rights=access_rights,
            partner_id=partner_id,
            text_reuse_cluster_id=text_reuse_cluster_id,
        )

        filters_pb = filters_as_protobuf(filters or [])

        result = get_search_facet.sync(
            client=self._api_client,
            id=facet_id,
            filters=filters_pb if filters_pb else UNSET,
            offset=offset if offset is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, GetSearchFacetOrderBy)
                if order_by is not None
                else UNSET
            ),
        )
        raise_for_error(result)
        return FacetDataContainer(
            result,
            SearchFacet,
            limit=limit,
            offset=offset,
            web_app_search_result_url=_build_web_app_facet_url(
                f"{self._get_web_app_base_url()}/search",
                facet=facet,
                filters=filters_pb,
                limit=limit,
                offset=offset,
            ),
        )

    def _build_filters(
        self,
        string: str | AND[str] | OR[str] | None,
        with_text_contents: bool | None = False,
        title: str | AND[str] | OR[str] | None = None,
        front_page: bool | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        newspaper_id: str | OR[str] | None = None,
        date_range: DateRange | None = None,
        language: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        access_rights: (
            ContentItemAccessRightLiteral | OR[ContentItemAccessRightLiteral] | None
        ) = None,
        partner_id: str | OR[str] | None = None,
        text_reuse_cluster_id: str | OR[str] | None = None,
    ) -> list[Filter]:
        filters: list[Filter] = []
        if string:
            filters.extend(and_or_filter(string, "string"))
        if with_text_contents:
            filters.append(Filter(type="has_text_contents", daterange=None))
        if title is not None:
            filters.extend(and_or_filter(title, "title"))
        if front_page:
            filters.append(Filter(type="is_front", daterange=None))
        if entity_id is not None:
            filters.extend(and_or_filter(entity_id, "entity"))
        if newspaper_id is not None:
            filters.extend(and_or_filter(newspaper_id, "newspaper"))
        if date_range is not None:
            filters.append(
                Filter(
                    type="daterange",
                    q=Q(DateRange._as_filter_value(date_range)),
                    context="exclude" if date_range.inverted else "include",
                    daterange=None,
                )
            )
        if language is not None:
            filters.extend(and_or_filter(language, "language"))
        if mention is not None:
            filters.extend(and_or_filter(mention, "mention"))
        if topic_id is not None:
            filters.extend(and_or_filter(topic_id, "topic"))
        if collection_id is not None:
            filters.extend(and_or_filter(collection_id, "collection"))
        if country is not None:
            filters.extend(and_or_filter(country, "country"))
        if access_rights is not None:
            filters.extend(and_or_filter(access_rights, "access_right"))  # type: ignore
        if partner_id is not None:
            filters.extend(and_or_filter(partner_id, "partner"))
        if text_reuse_cluster_id is not None:
            filters.extend(and_or_filter(text_reuse_cluster_id, "text_reuse_cluster"))

        return filters


def _build_web_app_search_url(
    base_url: str,
    order_by: SearchOrderByLiteral | None = None,
    filters: str | None = None,
    limit: int | None = None,
    offset: int | None = None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "orderBy": order_by,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    return f"{base_url}?{query_string}" if query_string else base_url


def _build_web_app_facet_url(
    base_url: str,
    facet: str,
    filters: str | None = None,
    limit: int | None = None,
    offset: int | None = None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "index": "search",
        "facet": "type",
        "domain": facet,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    return f"{base_url}?{query_string}" if query_string else base_url


def render_dataframe_chart_base64(x, y) -> str:
    plt.figure(figsize=(12, 1))

    if len(x) < 50:
        plt.bar(x, y)
    else:
        plt.plot(x, y)

    # Remove axes, labels, and legend
    plt.axis("off")
    # plt.legend().remove()

    # Save the plot to a bytes buffer with a transparent background
    buffer = io.BytesIO()
    plt.savefig(
        buffer, format="png", bbox_inches="tight", pad_inches=0.1, transparent=True
    )
    buffer.seek(0)

    # Encode the bytes as base64
    image_base64 = base64.b64encode(buffer.getvalue()).decode("utf-8")

    # Close the plot to free up memory
    plt.close()

    return image_base64



# generated by datamodel-codegen:
#   filename:  http://localhost:3030/swagger.json

from __future__ import annotations

from datetime import date
from typing import Any, Mapping, Optional, Sequence, Union

from pydantic import AnyUrl, AwareDatetime, BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated, Literal


class AuthenticationCreateRequest(BaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    strategy: Literal['local', 'jwt-app']
    email: Optional[str] = None
    password: Optional[str] = None
    accessToken: Optional[str] = None


class Authentication(BaseModel):
    strategy: Optional[str] = None
    payload: Optional[Mapping[str, Any]] = None


class BaseFind(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    limit: Annotated[
        int, Field(description='The number of items returned in this response')
    ]
    offset: Annotated[
        int,
        Field(
            description='Starting index of the items subset returned in this response'
        ),
    ]
    total: Annotated[
        int, Field(description='The total number of items matching the query')
    ]
    info: Annotated[
        Mapping[str, Any],
        Field(description='Additional information about the response.'),
    ]
    data: Sequence


class BaseUser(BaseModel):
    uid: Annotated[
        str,
        Field(
            examples=['local-dg'],
            pattern='^([a-zA-Z-]+)$',
            title='unique identifier for the user',
        ),
    ]
    username: Annotated[
        str,
        Field(
            examples=['daniele.guido'],
            pattern='^([a-z.]+)$',
            title='unique username for the user for other humans',
        ),
    ]


class CollectableItemsUpdatedResponse(BaseModel):
    totalAdded: Annotated[
        int, Field(description='Total number of items added to the collection')
    ]
    totalRemoved: Annotated[
        int, Field(description='Total number of items removed from the collection')
    ]


class Collection(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(max_length=50, min_length=2)]
    name: Annotated[str, Field(max_length=50, min_length=2)]
    description: Annotated[str, Field(max_length=500)]
    status: Annotated[
        str,
        Field(
            examples=['PRI'],
            max_length=3,
            min_length=2,
            title='Status of the collection',
        ),
    ]
    creationDate: AwareDatetime
    lastModifiedDate: AwareDatetime
    countItems: Annotated[
        Union[int, str], Field(title='Number of items in the collection')
    ]
    creator: BaseUser
    labels: Optional[Sequence[str]] = None


class Params(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[Optional[str], Field(None, description='The collection id')]
    status: Annotated[
        Optional[Literal['DEL']], Field(None, description='The status of the operation')
    ]


class Task(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    task_id: Annotated[Optional[str], Field(None, description='The ID of the task')]
    creationDate: Annotated[
        Optional[str], Field(None, description='When task was created')
    ]


class CollectionsRemoveResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    params: Params
    task: Annotated[Task, Field(description='Deletion task details')]


class PersonItem(RootModel[Sequence[Any]]):
    root: Annotated[Sequence[Any], Field(max_length=2, min_length=2)]


class LocationItem(RootModel[Sequence[Any]]):
    root: Annotated[Sequence[Any], Field(max_length=2, min_length=2)]


class Mention(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    person: Optional[Sequence[PersonItem]] = None
    location: Optional[Sequence[LocationItem]] = None


class ContentItemMatch(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    fragment: Annotated[str, Field(description='TODO')]
    coords: Annotated[Optional[Sequence[float]], Field(None, description='TODO')]
    pageUid: Annotated[Optional[str], Field(None, description='TODO')]
    iiif: Annotated[Optional[str], Field(None, description='TODO')]


class ContentItemRegion(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    pageUid: str
    coords: Sequence[float]
    isEmpty: Annotated[bool, Field(description='TODO')]
    iiifFragment: Annotated[Optional[str], Field(None, description='IIIF fragment URL')]
    g: Annotated[Optional[Sequence[str]], Field(None, description='TODO')]


class Entity(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique identifier of the entity')]
    relevance: Annotated[
        int, Field(description='Relevance of the entity in the document')
    ]


class Error(BaseModel):
    type: Annotated[
        AnyUrl,
        Field(
            description='A URI reference [RFC3986] that identifies the problem type.'
        ),
    ]
    title: Annotated[
        str, Field(description='A short, human-readable summary of the problem type.')
    ]
    status: Annotated[
        int, Field(description='The HTTP status code ([RFC7231], Section 6)')
    ]
    detail: Annotated[
        Optional[str],
        Field(
            None,
            description='A human-readable explanation specific to this occurrence of the problem.',
        ),
    ]


class Q(RootModel[str]):
    root: Annotated[str, Field(max_length=500, min_length=2)]


class QItem(RootModel[str]):
    root: Annotated[str, Field(max_length=500, min_length=2)]


class Filter(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    context: Optional[Literal['include', 'exclude']] = 'include'
    op: Optional[Literal['AND', 'OR']] = 'OR'
    type: Annotated[
        str,
        Field(
            description="Possible values are in 'search.validators:eachFilterValidator.type.choices'"
        ),
    ]
    precision: Optional[Literal['fuzzy', 'soft', 'exact', 'partial']] = 'exact'
    q: Optional[Union[Q, Sequence[QItem]]] = None
    daterange: Annotated[
        Optional[str],
        Field(
            None,
            pattern='\\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}Z TO \\d{4}-\\d{2}-\\d{2}T\\d{2}:\\d{2}:\\d{2}Z',
        ),
    ]
    uids: Optional[str] = None
    uid: Optional[str] = None


class Offset(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    start: Annotated[int, Field(description='Start offset of the entity in the text')]
    end: Annotated[int, Field(description='End offset of the entity in the text')]


class Confidence(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    ner: Annotated[
        float, Field(description='Confidence score for the named entity recognition')
    ]
    nel: Annotated[
        Optional[float],
        Field(None, description='Confidence score for the named entity linking'),
    ]


class Wikidata(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='Wikidata ID of the entity')]
    wikipediaPageName: Annotated[
        Optional[str], Field(None, description='Wikipedia page name of the entity')
    ]


class ImpressoNerEntity(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='ID of the entity')]
    type: Annotated[
        Literal[
            'comp.demonym',
            'comp.function',
            'comp.name',
            'comp.qualifier',
            'comp.title',
            'loc',
            'loc.add.elec',
            'loc.add.phys',
            'loc.adm.nat',
            'loc.adm.reg',
            'loc.adm.sup',
            'loc.adm.town',
            'loc.fac',
            'loc.oro',
            'loc.phys.astro',
            'loc.phys.geo',
            'loc.phys.hydro',
            'loc.unk',
            'org',
            'org.adm',
            'org.ent',
            'org.ent.pressagency',
            'pers',
            'pers.coll',
            'pers.ind',
            'pers.ind.articleauthor',
            'prod',
            'prod.doctr',
            'prod.media',
            'time',
            'time.date.abs',
            'time.hour.abs',
        ],
        Field(description='Type of the entity'),
    ]
    surfaceForm: Annotated[str, Field(description='Surface form of the entity')]
    offset: Offset
    isTypeNested: Annotated[
        Optional[bool], Field(None, description='Whether the entity type is nested')
    ]
    confidence: Confidence
    wikidata: Optional[Wikidata] = None
    function: Annotated[
        Optional[str], Field(None, description='Function of the entity')
    ]
    name: Annotated[Optional[str], Field(None, description='Name of the entity')]


class ImpressoNerRequest(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    text: Annotated[
        str,
        Field(
            description='Text to be processed for named entity recognition',
            max_length=3999,
            min_length=1,
        ),
    ]


class ImpressoNerResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    modelId: Annotated[
        str, Field(description='ID of the model used for the named entity recognition')
    ]
    text: Annotated[
        str, Field(description='Text processed for named entity recognition')
    ]
    timestamp: Annotated[
        AwareDatetime,
        Field(description='Timestamp of when named entity recognition was performed'),
    ]
    entities: Sequence[ImpressoNerEntity]


class NewCollection(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    name: Annotated[str, Field(max_length=50, min_length=2)]
    description: Annotated[Optional[str], Field(None, max_length=500)]
    status: Annotated[
        Optional[str],
        Field(
            None,
            examples=['PRI'],
            max_length=3,
            min_length=2,
            title='Status of the collection',
        ),
    ]


class NewspaperIssue(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the issue')]
    cover: Annotated[str, Field(description='TODO')]
    labels: Annotated[Sequence[str], Field(description='The labels of the issue')]
    fresh: Annotated[bool, Field(description='TODO')]
    accessRights: Annotated[str, Field(description='TODO: list available options')]
    date: Annotated[
        Optional[AwareDatetime], Field(None, description='The date of the issue')
    ]
    year: Annotated[Optional[str], Field(None, description='The year of the issue')]


class NewspaperProperty(BaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    name: Annotated[str, Field(description='The name of the property')]
    value: Annotated[str, Field(description='The value of the property')]
    label: Annotated[str, Field(description='The label of the property')]
    isUrl: Annotated[
        Optional[bool], Field(None, description='Whether the value is a URL')
    ]


class Page(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the page')]
    num: Annotated[int, Field(description='The number of the page')]
    issueUid: Annotated[str, Field(description='Reference to the article')]
    newspaperUid: Annotated[str, Field(description='Unique ID of the newspaper')]
    iiif: Annotated[str, Field(description='The IIF image file name of the page')]
    iiifThumbnail: Annotated[
        str, Field(description='The IIF image thumbnail file name of the page')
    ]
    accessRights: Annotated[str, Field(description='The access rights code')]
    labels: Annotated[Sequence[str], Field(description='Page labels')]
    hasCoords: Annotated[bool, Field(description='Whether the page has coordinates')]
    hasErrors: Annotated[bool, Field(description='Whether the page has errors')]
    regions: Annotated[
        Sequence[Mapping[str, Any]], Field(description='Regions of the page')
    ]
    obfuscated: Annotated[
        Optional[bool],
        Field(
            None,
            description='Whether the page image has been obfuscated because the user is not authorised to access it',
        ),
    ]
    iiifFragment: Annotated[
        Optional[str],
        Field(None, description='The IIIF fragment of the page, image file name'),
    ]


class SearchFacetRangeBucket(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    count: Annotated[int, Field(description='Number of items in the bucket')]
    val: Annotated[int, Field(description="Value of the 'type' element")]
    lower: Annotated[Optional[int], Field(None, description='Lower bound of the range')]
    upper: Annotated[Optional[int], Field(None, description='Lower bound of the range')]


class TimeCoverage(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    from_: Annotated[Optional[date], Field(None, alias='from')]
    to: Optional[date] = None


class TextReuseCluster(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[
        str,
        Field(
            description='ID of the text reuse passage',
            examples=['abc123'],
            pattern='^[a-zA-Z0-9-_]+$',
            title='Passage ID',
        ),
    ]
    lexicalOverlap: Annotated[
        Optional[float],
        Field(
            None,
            description='Percentage of overlap between passages in the cluster',
            ge=0.0,
            le=100.0,
        ),
    ]
    clusterSize: Annotated[
        Optional[float],
        Field(None, description='Number of passages in cluster', ge=0.0),
    ]
    connectedClustersCount: Annotated[
        Optional[float], Field(None, description='Number of connected clusters', ge=0.0)
    ]
    timeCoverage: Annotated[
        Optional[TimeCoverage],
        Field(None, description='Time window covered by documents in the cluster'),
    ]


class Facet(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    type: Annotated[Optional[str], Field(None, description='Facet type')]
    numBuckets: Annotated[Optional[int], Field(None, description='Number of buckets')]
    buckets: Optional[Sequence[Mapping[str, Any]]] = None


class TextReuseClusterDetails(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    facets: Sequence[Facet]
    resolution: Annotated[
        Optional[Literal['year', 'month', 'day']],
        Field(None, description="Resolution for the 'date' facet"),
    ]


class Article(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='ID of the article', title='Article ID')]


class TextReuseCluster1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='ID of the cluster', title='Cluster ID')]
    clusterSize: Annotated[
        Optional[int],
        Field(None, description='The size of the cluster', title='Cluster size'),
    ]
    timeDifferenceDay: Annotated[
        Optional[int],
        Field(
            None,
            description='The time difference in days between the two articles',
            title='Time difference in days',
        ),
    ]
    lexicalOverlap: Annotated[
        Optional[float],
        Field(
            None,
            description='The lexical overlap between the two articles',
            title='Lexical overlap',
        ),
    ]


class OffsetStart(RootModel[Optional[int]]):
    root: Annotated[
        Optional[int],
        Field(None, description='Offset of the passage in the article text', ge=0),
    ] = None


class OffsetEnd(RootModel[Optional[int]]):
    root: Annotated[
        Optional[int],
        Field(None, description='Offset of the passage in the article text', ge=0),
    ] = None


class ConnectedCluster(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='ID of the connected cluster')]


class Issue(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[str, Field(description='ID of the issue')]


class TextReusePassage(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: Annotated[
        str,
        Field(
            description='ID of the text reuse passage',
            examples=['abc123'],
            pattern='^[a-zA-Z0-9-_@:]+$',
            title='Passage ID',
        ),
    ]
    article: Annotated[
        Article,
        Field(
            description='Details of the article the passage belongs to',
            title='Article details',
        ),
    ]
    textReuseCluster: Annotated[
        TextReuseCluster1,
        Field(
            description='Details of the cluster the passage belongs to',
            title='Cluster details',
        ),
    ]
    offsetStart: OffsetStart
    offsetEnd: OffsetEnd
    content: Annotated[str, Field(description='Textual content of the passage')]
    title: Annotated[
        str,
        Field(
            description='Title of the content item (article) where this passage was found'
        ),
    ]
    connectedClusters: Optional[Sequence[ConnectedCluster]] = None
    isFront: Annotated[Optional[bool], Field(None, description='TBD')]
    size: Annotated[Optional[int], Field(None, description='Size of the passage')]
    newspaper: Optional[Any] = None
    issue: Annotated[Optional[Issue], Field(None, description='Issue details')]
    date: Annotated[
        Optional[AwareDatetime],
        Field(
            None, description='Date of the item (article) where this passage was found'
        ),
    ]
    pageRegions: Annotated[
        Optional[Sequence[str]],
        Field(None, description='Bounding box of the passage in the page'),
    ]
    pageNumbers: Annotated[
        Sequence[int],
        Field(description='Numbers of the pages where the passage was found'),
    ]
    collections: Annotated[
        Sequence[str], Field(description='Collection IDs the passage belongs to')
    ]


class RelatedTopic(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the related topic')]
    w: Annotated[float, Field(description='TODO')]
    avg: Annotated[Optional[float], Field(None, description='TODO')]


class TopicWord(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    w: Annotated[str, Field(description='Word')]
    p: Annotated[float, Field(description='TODO')]
    h: Annotated[Optional[Sequence[str]], Field(None, description='TODO')]


class UpdateCollectableItems(BaseModel):
    add: Annotated[
        Optional[Sequence[str]],
        Field(None, description='IDs of the items to add to the collection'),
    ]
    remove: Annotated[
        Optional[Sequence[str]],
        Field(None, description='IDs of the items to remove from the collection'),
    ]


class User(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    id: int
    username: str
    firstname: str
    lastname: str
    isStaff: bool
    isActive: bool
    isSuperuser: bool
    uid: str


class Solr(BaseModel):
    endpoints: Optional[Mapping[str, str]] = None


class Mysql(BaseModel):
    endpoint: Optional[str] = None


class ApiVersion(BaseModel):
    branch: Optional[str] = None
    revision: Optional[str] = None
    version: Optional[str] = None


class DocumentsDateSpan(BaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    start: Optional[AwareDatetime] = None
    end: Optional[AwareDatetime] = None


class Newspapers(BaseModel):
    name: Optional[str] = None


class VersionDetails(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    solr: Solr
    mysql: Mysql
    version: str
    apiVersion: ApiVersion
    documentsDateSpan: DocumentsDateSpan
    newspapers: Mapping[str, Newspapers]
    features: Mapping[str, Mapping[str, Any]]


class Image(BaseModel):
    value: str
    rank: str
    datatype: str


class WikidataEntityDetails(BaseModel):
    model_config = ConfigDict(
        extra='allow',
    )
    id: str
    type: str
    labels: Annotated[
        Mapping[str, str],
        Field(description='Labels of the entity. Key is the language code.'),
    ]
    descriptions: Annotated[
        Mapping[str, str],
        Field(description='Labels of the entity. Key is the language code.'),
    ]
    images: Sequence[Image]


class YearWeights(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    c: Annotated[Optional[float], Field(None, description='Number of content items')]
    a: Annotated[Optional[float], Field(None, description='Number of articles')]
    p: Annotated[Optional[float], Field(None, description='Number of pages')]
    i: Annotated[Optional[float], Field(None, description='Number of issues')]
    m: Annotated[
        Optional[float],
        Field(None, description='Number of images (with or without vectors)'),
    ]


class AuthenticationCreateResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    accessToken: str
    authentication: Authentication
    user: User


class CollectableItemGroup(BaseModel):
    itemId: Annotated[
        Optional[str], Field(None, description='The id of the collectable item group')
    ]
    contentType: Annotated[
        Optional[Literal['A', 'E', 'P', 'I']],
        Field(
            None,
            description='Content type of the collectable item group: (A)rticle, (E)ntities, (P)ages, (I)ssues',
        ),
    ]
    collectionIds: Annotated[
        Optional[Sequence[str]], Field(None, description='Ids of the collections')
    ]
    searchQueries: Annotated[
        Optional[Sequence[str]], Field(None, description='Search queries')
    ]
    collections: Annotated[
        Optional[Sequence[Collection]], Field(None, description='Collection objects')
    ]
    latestDateAdded: Annotated[
        Optional[AwareDatetime],
        Field(None, description='The latest date added to the collectable item group'),
    ]


class EntityDetails(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='Unique identifier of the entity')]
    name: Annotated[str, Field(description='Entity name')]
    type: Literal['person', 'location']
    countItems: Annotated[int, Field(description='TODO')]
    countMentions: Annotated[
        int, Field(description='Number of mentions of this entity in articles')
    ]
    wikidataId: Annotated[
        Optional[str], Field(None, description='ID of the entity in wikidata')
    ]
    wikidata: Optional[WikidataEntityDetails] = None


class Newspaper(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the newspaper')]
    acronym: Annotated[str, Field(description='The acronym of the newspaper')]
    labels: Annotated[Sequence[str], Field(description='The labels of the newspaper')]
    languages: Annotated[
        Sequence[str],
        Field(description='Language codes of the languages used in the newspaper'),
    ]
    properties: Annotated[
        Optional[Sequence[NewspaperProperty]], Field(None, description='TODO')
    ]
    included: Annotated[bool, Field(description='TODO')]
    name: Annotated[str, Field(description='Title of the newspaper')]
    endYear: int
    startYear: int
    firstIssue: Optional[NewspaperIssue] = None
    lastIssue: Optional[NewspaperIssue] = None
    countArticles: Annotated[
        int, Field(description='The number of articles in the newspaper')
    ]
    countIssues: Annotated[
        int, Field(description='The number of issues in the newspaper')
    ]
    countPages: Annotated[
        int, Field(description='The number of pages in the newspaper')
    ]
    fetched: Annotated[Optional[bool], Field(None, description='TODO')]
    deltaYear: Annotated[
        int, Field(description='The number of years of the newspaper available')
    ]


class TextReuseClusterCompound(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    cluster: Optional[TextReuseCluster] = None
    textSample: str
    details: Optional[TextReuseClusterDetails] = None


class Topic(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the topic')]
    language: Annotated[str, Field(description='The language code of the topic')]
    community: Annotated[Optional[str], Field(None, description='TODO')]
    pagerank: Annotated[Optional[float], Field(None, description='TODO')]
    degree: Annotated[Optional[float], Field(None, description='TODO')]
    x: Annotated[Optional[float], Field(None, description='TODO')]
    y: Annotated[Optional[float], Field(None, description='TODO')]
    relatedTopics: Optional[Sequence[RelatedTopic]] = None
    countItems: Annotated[Optional[float], Field(None, description='TODO')]
    excerpt: Annotated[Optional[Sequence[TopicWord]], Field(None, description='TODO')]
    words: Annotated[Optional[Sequence[TopicWord]], Field(None, description='TODO')]
    model: Annotated[
        Optional[str],
        Field(None, description='ID of the model used to generate the topic'),
    ]


class Year(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[
        Optional[int], Field(None, description='Numeric representation of the year')
    ]
    values: Optional[YearWeights] = None
    refs: Optional[YearWeights] = None


class ContentItemTopic(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    topic: Optional[Topic] = None
    relevance: Annotated[float, Field(description='TODO')]
    topicUid: Annotated[Optional[str], Field(None, description='TODO')]


class FindTextReuseClustersResponse(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    clusters: Sequence[TextReuseClusterCompound]
    info: Any


class SearchFacetBucket(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    count: Annotated[int, Field(description='Number of items in the bucket')]
    val: Annotated[str, Field(description="Value of the 'type' element")]
    uid: Annotated[
        Optional[str],
        Field(None, description="UID of the 'type' element. Same as 'val'"),
    ]
    item: Annotated[
        Optional[Union[Newspaper, Collection, Entity, Topic, Year]],
        Field(
            None,
            description='The item in the bucket. Particular objct schema depends on the facet type',
        ),
    ]


class ContentItem(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    uid: Annotated[str, Field(description='The unique identifier of the content item')]
    type: Annotated[
        str, Field(description='The type of the content item. NOTE: may be empty.')
    ]
    title: Annotated[str, Field(description='The title of the content item')]
    size: Annotated[
        int, Field(description='The size of the content item in characters')
    ]
    nbPages: Annotated[
        int, Field(description='The number of pages in this content item')
    ]
    pages: Sequence[Page]
    isCC: Annotated[bool, Field(description='TODO')]
    excerpt: Annotated[str, Field(description='The excerpt of the content item')]
    locations: Optional[Sequence[Entity]] = None
    persons: Optional[Sequence[Entity]] = None
    language: Annotated[
        Optional[str], Field(None, description='The language code of the content item')
    ]
    issue: Optional[NewspaperIssue] = None
    matches: Optional[Sequence[ContentItemMatch]] = None
    regions: Optional[Sequence[ContentItemRegion]] = None
    regionBreaks: Optional[Sequence[int]] = None
    contentLineBreaks: Optional[Sequence[int]] = None
    labels: Annotated[Sequence[Literal['article']], Field(description='TODO')]
    accessRight: Literal['na', 'OpenPrivate', 'Closed', 'OpenPublic']
    isFront: Annotated[Optional[bool], Field(None, description='TODO')]
    date: Optional[AwareDatetime] = None
    year: Annotated[int, Field(description='The year of the content item')]
    country: Annotated[
        Optional[str], Field(None, description='The country code of the content item')
    ]
    tags: Optional[Sequence[str]] = None
    collections: Optional[Union[Sequence[str], Sequence[Collection]]] = None
    newspaper: Optional[Newspaper] = None
    dataProvider: Optional[str] = None
    topics: Optional[Sequence[ContentItemTopic]] = None
    content: Annotated[
        Optional[str], Field(None, description='The content of the content item')
    ]
    mentions: Optional[Sequence[Mention]] = None
    v: Annotated[Optional[str], Field(None, description='TODO')]


class SearchFacet(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    type: Annotated[str, Field(description='The type of facet')]
    numBuckets: Annotated[int, Field(description='The number of buckets in the facet')]
    buckets: Union[Sequence[SearchFacetBucket], Sequence[SearchFacetRangeBucket]]
    min: Annotated[Optional[Any], Field(None, description='TODO')]
    max: Annotated[Optional[Any], Field(None, description='TODO')]
    gap: Annotated[Optional[Any], Field(None, description='TODO')]



"""Contains all the data models used in inputs/outputs"""

from .api_version import APIVersion
from .api_version_api_version import APIVersionApiVersion
from .api_version_documents_date_span import APIVersionDocumentsDateSpan
from .api_version_features import APIVersionFeatures
from .api_version_features_additional_property import APIVersionFeaturesAdditionalProperty
from .api_version_mysql import APIVersionMysql
from .api_version_newspapers import APIVersionNewspapers
from .api_version_newspapers_additional_property import APIVersionNewspapersAdditionalProperty
from .api_version_solr import APIVersionSolr
from .api_version_solr_endpoints import APIVersionSolrEndpoints
from .authentication_create_request import AuthenticationCreateRequest
from .authentication_create_request_strategy import AuthenticationCreateRequestStrategy
from .authentication_response import AuthenticationResponse
from .authentication_response_authentication import AuthenticationResponseAuthentication
from .authentication_response_authentication_payload import AuthenticationResponseAuthenticationPayload
from .base_find import BaseFind
from .base_find_info import BaseFindInfo
from .base_user import BaseUser
from .collectable_item_group import CollectableItemGroup
from .collectable_item_group_content_type import CollectableItemGroupContentType
from .collectable_items_updated_response import CollectableItemsUpdatedResponse
from .collection import Collection
from .content_item import ContentItem
from .content_item_access_right import ContentItemAccessRight
from .content_item_labels_item import ContentItemLabelsItem
from .content_item_match import ContentItemMatch
from .content_item_mentions_item import ContentItemMentionsItem
from .content_item_region import ContentItemRegion
from .content_item_topic import ContentItemTopic
from .entity import Entity
from .entity_details import EntityDetails
from .entity_details_type import EntityDetailsType
from .error import Error
from .filter_ import Filter
from .filter_context import FilterContext
from .filter_op import FilterOp
from .filter_precision import FilterPrecision
from .find_collections_order_by import FindCollectionsOrderBy
from .find_collections_response_200 import FindCollectionsResponse200
from .find_collections_response_200_info import FindCollectionsResponse200Info
from .find_content_item_order_by import FindContentItemOrderBy
from .find_content_item_resolve import FindContentItemResolve
from .find_content_item_response_200 import FindContentItemResponse200
from .find_content_item_response_200_info import FindContentItemResponse200Info
from .find_entities_order_by import FindEntitiesOrderBy
from .find_entities_response_200 import FindEntitiesResponse200
from .find_entities_response_200_info import FindEntitiesResponse200Info
from .find_newspapers_order_by import FindNewspapersOrderBy
from .find_newspapers_response_200 import FindNewspapersResponse200
from .find_newspapers_response_200_info import FindNewspapersResponse200Info
from .find_search_facets_facets_item import FindSearchFacetsFacetsItem
from .find_search_facets_group_by import FindSearchFacetsGroupBy
from .find_search_facets_order_by import FindSearchFacetsOrderBy
from .find_search_facets_range_include import FindSearchFacetsRangeInclude
from .find_search_facets_response_200 import FindSearchFacetsResponse200
from .find_search_facets_response_200_info import FindSearchFacetsResponse200Info
from .find_text_reuse_clusters_order_by import FindTextReuseClustersOrderBy
from .find_text_reuse_clusters_response import FindTextReuseClustersResponse
from .find_text_reuse_passages_addons import FindTextReusePassagesAddons
from .find_text_reuse_passages_group_by import FindTextReusePassagesGroupBy
from .find_text_reuse_passages_order_by import FindTextReusePassagesOrderBy
from .find_text_reuse_passages_response_200 import FindTextReusePassagesResponse200
from .find_text_reuse_passages_response_200_info import FindTextReusePassagesResponse200Info
from .find_tr_clusters_facets_facets_item import FindTrClustersFacetsFacetsItem
from .find_tr_clusters_facets_group_by import FindTrClustersFacetsGroupBy
from .find_tr_clusters_facets_order_by import FindTrClustersFacetsOrderBy
from .find_tr_clusters_facets_range_include import FindTrClustersFacetsRangeInclude
from .find_tr_clusters_facets_response_200 import FindTrClustersFacetsResponse200
from .find_tr_clusters_facets_response_200_info import FindTrClustersFacetsResponse200Info
from .find_tr_passages_facets_facets_item import FindTrPassagesFacetsFacetsItem
from .find_tr_passages_facets_group_by import FindTrPassagesFacetsGroupBy
from .find_tr_passages_facets_order_by import FindTrPassagesFacetsOrderBy
from .find_tr_passages_facets_range_include import FindTrPassagesFacetsRangeInclude
from .find_tr_passages_facets_response_200 import FindTrPassagesFacetsResponse200
from .find_tr_passages_facets_response_200_info import FindTrPassagesFacetsResponse200Info
from .get_search_facet_group_by import GetSearchFacetGroupBy
from .get_search_facet_id import GetSearchFacetId
from .get_search_facet_order_by import GetSearchFacetOrderBy
from .get_search_facet_range_include import GetSearchFacetRangeInclude
from .get_tr_clusters_facet_group_by import GetTrClustersFacetGroupBy
from .get_tr_clusters_facet_id import GetTrClustersFacetId
from .get_tr_clusters_facet_order_by import GetTrClustersFacetOrderBy
from .get_tr_clusters_facet_range_include import GetTrClustersFacetRangeInclude
from .get_tr_passages_facet_group_by import GetTrPassagesFacetGroupBy
from .get_tr_passages_facet_id import GetTrPassagesFacetId
from .get_tr_passages_facet_order_by import GetTrPassagesFacetOrderBy
from .get_tr_passages_facet_range_include import GetTrPassagesFacetRangeInclude
from .impresso_named_entity_recognition_entity import ImpressoNamedEntityRecognitionEntity
from .impresso_named_entity_recognition_entity_confidence import ImpressoNamedEntityRecognitionEntityConfidence
from .impresso_named_entity_recognition_entity_offset import ImpressoNamedEntityRecognitionEntityOffset
from .impresso_named_entity_recognition_entity_type import ImpressoNamedEntityRecognitionEntityType
from .impresso_named_entity_recognition_entity_wikidata import ImpressoNamedEntityRecognitionEntityWikidata
from .impresso_named_entity_recognition_request import ImpressoNamedEntityRecognitionRequest
from .impresso_named_entity_recognition_response import ImpressoNamedEntityRecognitionResponse
from .location_specific_fields import LocationSpecificFields
from .location_specific_fields_descriptions import LocationSpecificFieldsDescriptions
from .location_specific_fields_images_item import LocationSpecificFieldsImagesItem
from .location_specific_fields_labels import LocationSpecificFieldsLabels
from .new_collection import NewCollection
from .newspaper import Newspaper
from .newspaper_issue import NewspaperIssue
from .newspaper_property import NewspaperProperty
from .page import Page
from .page_regions_item import PageRegionsItem
from .remove_collection_response import RemoveCollectionResponse
from .remove_collection_response_params import RemoveCollectionResponseParams
from .remove_collection_response_params_status import RemoveCollectionResponseParamsStatus
from .remove_collection_response_task import RemoveCollectionResponseTask
from .search_facet import SearchFacet
from .search_facet_bucket import SearchFacetBucket
from .search_facet_range_bucket import SearchFacetRangeBucket
from .search_facets import SearchFacets
from .search_group_by import SearchGroupBy
from .search_order_by import SearchOrderBy
from .search_response_200 import SearchResponse200
from .search_response_200_info import SearchResponse200Info
from .text_reuse_cluster import TextReuseCluster
from .text_reuse_cluster_compound import TextReuseClusterCompound
from .text_reuse_cluster_details import TextReuseClusterDetails
from .text_reuse_cluster_details_facets_item import TextReuseClusterDetailsFacetsItem
from .text_reuse_cluster_details_facets_item_buckets_item import TextReuseClusterDetailsFacetsItemBucketsItem
from .text_reuse_cluster_details_resolution import TextReuseClusterDetailsResolution
from .text_reuse_cluster_time_coverage import TextReuseClusterTimeCoverage
from .text_reuse_passage import TextReusePassage
from .text_reuse_passage_article_details import TextReusePassageArticleDetails
from .text_reuse_passage_cluster_details import TextReusePassageClusterDetails
from .text_reuse_passage_connected_clusters_item import TextReusePassageConnectedClustersItem
from .text_reuse_passage_issue import TextReusePassageIssue
from .topic import Topic
from .topic_related_topics_item import TopicRelatedTopicsItem
from .topic_word import TopicWord
from .update_collectable_items import UpdateCollectableItems
from .user import User
from .year import Year
from .year_weights import YearWeights

__all__ = (
    "APIVersion",
    "APIVersionApiVersion",
    "APIVersionDocumentsDateSpan",
    "APIVersionFeatures",
    "APIVersionFeaturesAdditionalProperty",
    "APIVersionMysql",
    "APIVersionNewspapers",
    "APIVersionNewspapersAdditionalProperty",
    "APIVersionSolr",
    "APIVersionSolrEndpoints",
    "AuthenticationCreateRequest",
    "AuthenticationCreateRequestStrategy",
    "AuthenticationResponse",
    "AuthenticationResponseAuthentication",
    "AuthenticationResponseAuthenticationPayload",
    "BaseFind",
    "BaseFindInfo",
    "BaseUser",
    "CollectableItemGroup",
    "CollectableItemGroupContentType",
    "CollectableItemsUpdatedResponse",
    "Collection",
    "ContentItem",
    "ContentItemAccessRight",
    "ContentItemLabelsItem",
    "ContentItemMatch",
    "ContentItemMentionsItem",
    "ContentItemRegion",
    "ContentItemTopic",
    "Entity",
    "EntityDetails",
    "EntityDetailsType",
    "Error",
    "Filter",
    "FilterContext",
    "FilterOp",
    "FilterPrecision",
    "FindCollectionsOrderBy",
    "FindCollectionsResponse200",
    "FindCollectionsResponse200Info",
    "FindContentItemOrderBy",
    "FindContentItemResolve",
    "FindContentItemResponse200",
    "FindContentItemResponse200Info",
    "FindEntitiesOrderBy",
    "FindEntitiesResponse200",
    "FindEntitiesResponse200Info",
    "FindNewspapersOrderBy",
    "FindNewspapersResponse200",
    "FindNewspapersResponse200Info",
    "FindSearchFacetsFacetsItem",
    "FindSearchFacetsGroupBy",
    "FindSearchFacetsOrderBy",
    "FindSearchFacetsRangeInclude",
    "FindSearchFacetsResponse200",
    "FindSearchFacetsResponse200Info",
    "FindTextReuseClustersOrderBy",
    "FindTextReuseClustersResponse",
    "FindTextReusePassagesAddons",
    "FindTextReusePassagesGroupBy",
    "FindTextReusePassagesOrderBy",
    "FindTextReusePassagesResponse200",
    "FindTextReusePassagesResponse200Info",
    "FindTrClustersFacetsFacetsItem",
    "FindTrClustersFacetsGroupBy",
    "FindTrClustersFacetsOrderBy",
    "FindTrClustersFacetsRangeInclude",
    "FindTrClustersFacetsResponse200",
    "FindTrClustersFacetsResponse200Info",
    "FindTrPassagesFacetsFacetsItem",
    "FindTrPassagesFacetsGroupBy",
    "FindTrPassagesFacetsOrderBy",
    "FindTrPassagesFacetsRangeInclude",
    "FindTrPassagesFacetsResponse200",
    "FindTrPassagesFacetsResponse200Info",
    "GetSearchFacetGroupBy",
    "GetSearchFacetId",
    "GetSearchFacetOrderBy",
    "GetSearchFacetRangeInclude",
    "GetTrClustersFacetGroupBy",
    "GetTrClustersFacetId",
    "GetTrClustersFacetOrderBy",
    "GetTrClustersFacetRangeInclude",
    "GetTrPassagesFacetGroupBy",
    "GetTrPassagesFacetId",
    "GetTrPassagesFacetOrderBy",
    "GetTrPassagesFacetRangeInclude",
    "ImpressoNamedEntityRecognitionEntity",
    "ImpressoNamedEntityRecognitionEntityConfidence",
    "ImpressoNamedEntityRecognitionEntityOffset",
    "ImpressoNamedEntityRecognitionEntityType",
    "ImpressoNamedEntityRecognitionEntityWikidata",
    "ImpressoNamedEntityRecognitionRequest",
    "ImpressoNamedEntityRecognitionResponse",
    "LocationSpecificFields",
    "LocationSpecificFieldsDescriptions",
    "LocationSpecificFieldsImagesItem",
    "LocationSpecificFieldsLabels",
    "NewCollection",
    "Newspaper",
    "NewspaperIssue",
    "NewspaperProperty",
    "Page",
    "PageRegionsItem",
    "RemoveCollectionResponse",
    "RemoveCollectionResponseParams",
    "RemoveCollectionResponseParamsStatus",
    "RemoveCollectionResponseTask",
    "SearchFacet",
    "SearchFacetBucket",
    "SearchFacetRangeBucket",
    "SearchFacets",
    "SearchGroupBy",
    "SearchOrderBy",
    "SearchResponse200",
    "SearchResponse200Info",
    "TextReuseCluster",
    "TextReuseClusterCompound",
    "TextReuseClusterDetails",
    "TextReuseClusterDetailsFacetsItem",
    "TextReuseClusterDetailsFacetsItemBucketsItem",
    "TextReuseClusterDetailsResolution",
    "TextReuseClusterTimeCoverage",
    "TextReusePassage",
    "TextReusePassageArticleDetails",
    "TextReusePassageClusterDetails",
    "TextReusePassageConnectedClustersItem",
    "TextReusePassageIssue",
    "Topic",
    "TopicRelatedTopicsItem",
    "TopicWord",
    "UpdateCollectableItems",
    "User",
    "Year",
    "YearWeights",
)



from typing import Any, Generic, TypeVar
from pydantic import BaseModel
from pandas import DataFrame

IT = TypeVar("IT")
T = TypeVar("T", bound=BaseModel)


class DataContainer(Generic[IT, T]):
    """Response of a resource call"""

    def __init__(
        self,
        data: IT,
        pydantic_model: type[T],
        web_app_search_result_url: str | None = None,
    ):
        if data is None or getattr(data, "to_dict") is None:
            raise ValueError(f"Unexpected data object: {data}")
        self._data = data
        self._pydantic_model = pydantic_model
        self._web_app_search_result_url = web_app_search_result_url

    def _repr_html_(self):
        df_repr = self.df.head(3).to_html(notebook=True)
        response_type = self.__class__.__name__.replace("DataContainer", "").replace(
            "Container", ""
        )
        preview_img = self._get_preview_image_()

        items = [
            f"<h2>{response_type} result</h2>",
            f"<div>Contains <b>{self.size}</b> items "
            + (
                f"(<b>{self.offset}</b> - <b>{self.offset + self.size}</b>) "
                if self.size > 0 and self.size < self.total
                else ""
            )
            + f"of <b>{self.total}</b> total items.</div>",
            "<br/>",
            (
                f'See this result in the <a href="{self.url}">Impresso App</a>.'
                if self.url
                else None
            ),
            "<h3>Data preview:</h3>",
            df_repr,
            (
                f'<img src="data:image/png;base64,{preview_img}" style="width:100% !important;">'
                if preview_img
                else None
            ),
        ]

        return "\n".join([item for item in items if item])

    def _get_preview_image_(self) -> str | None:
        return None

    @property
    def raw(self) -> dict[str, Any]:
        """Return the data as a python dictionary."""
        return getattr(self._data, "to_dict")()

    @property
    def pydantic(self) -> T:
        """Return the data as a pydantic model."""
        return self._pydantic_model.model_validate(self.raw)

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        return DataFrame.from_dict(self._data)  # type: ignore

    @property
    def total(self) -> int:
        """Total number of results."""
        return self.raw.get("total", 0)

    @property
    def limit(self) -> int:
        """Page size."""
        return self.raw.get("limit", 0)

    @property
    def offset(self) -> int:
        """Page offset."""
        return self.raw.get("offset", 0)

    @property
    def size(self) -> int:
        """Current page size."""
        return len(self.raw.get("data", []))

    @property
    def url(self) -> str | None:
        """A URL of the result set in the Impresso web app."""
        return self._web_app_search_result_url



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.api_version_api_version import APIVersionApiVersion
    from ..models.api_version_documents_date_span import APIVersionDocumentsDateSpan
    from ..models.api_version_features import APIVersionFeatures
    from ..models.api_version_mysql import APIVersionMysql
    from ..models.api_version_newspapers import APIVersionNewspapers
    from ..models.api_version_solr import APIVersionSolr


T = TypeVar("T", bound="APIVersion")


@_attrs_define
class APIVersion:
    """Version of the API. Contains information about the current version of the API, features, etc.

    Attributes:
        solr (APIVersionSolr):
        mysql (APIVersionMysql):
        version (str):
        api_version (APIVersionApiVersion):
        documents_date_span (APIVersionDocumentsDateSpan):
        newspapers (APIVersionNewspapers):
        features (APIVersionFeatures):
    """

    solr: "APIVersionSolr"
    mysql: "APIVersionMysql"
    version: str
    api_version: "APIVersionApiVersion"
    documents_date_span: "APIVersionDocumentsDateSpan"
    newspapers: "APIVersionNewspapers"
    features: "APIVersionFeatures"

    def to_dict(self) -> Dict[str, Any]:
        solr = self.solr.to_dict()

        mysql = self.mysql.to_dict()

        version = self.version

        api_version = self.api_version.to_dict()

        documents_date_span = self.documents_date_span.to_dict()

        newspapers = self.newspapers.to_dict()

        features = self.features.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "solr": solr,
                "mysql": mysql,
                "version": version,
                "apiVersion": api_version,
                "documentsDateSpan": documents_date_span,
                "newspapers": newspapers,
                "features": features,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_version_api_version import APIVersionApiVersion
        from ..models.api_version_documents_date_span import APIVersionDocumentsDateSpan
        from ..models.api_version_features import APIVersionFeatures
        from ..models.api_version_mysql import APIVersionMysql
        from ..models.api_version_newspapers import APIVersionNewspapers
        from ..models.api_version_solr import APIVersionSolr

        d = src_dict.copy()
        solr = APIVersionSolr.from_dict(d.pop("solr"))

        mysql = APIVersionMysql.from_dict(d.pop("mysql"))

        version = d.pop("version")

        api_version = APIVersionApiVersion.from_dict(d.pop("apiVersion"))

        documents_date_span = APIVersionDocumentsDateSpan.from_dict(d.pop("documentsDateSpan"))

        newspapers = APIVersionNewspapers.from_dict(d.pop("newspapers"))

        features = APIVersionFeatures.from_dict(d.pop("features"))

        api_version = cls(
            solr=solr,
            mysql=mysql,
            version=version,
            api_version=api_version,
            documents_date_span=documents_date_span,
            newspapers=newspapers,
            features=features,
        )

        return api_version



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="CollectableItemsUpdatedResponse")


@_attrs_define
class CollectableItemsUpdatedResponse:
    """Request to update collectible items in a collection

    Attributes:
        total_added (int): Total number of items added to the collection
        total_removed (int): Total number of items removed from the collection
    """

    total_added: int
    total_removed: int
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total_added = self.total_added

        total_removed = self.total_removed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "totalAdded": total_added,
                "totalRemoved": total_removed,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total_added = d.pop("totalAdded")

        total_removed = d.pop("totalRemoved")

        collectable_items_updated_response = cls(
            total_added=total_added,
            total_removed=total_removed,
        )

        collectable_items_updated_response.additional_properties = d
        return collectable_items_updated_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_version_solr_endpoints import APIVersionSolrEndpoints


T = TypeVar("T", bound="APIVersionSolr")


@_attrs_define
class APIVersionSolr:
    """
    Attributes:
        endpoints (Union[Unset, APIVersionSolrEndpoints]):
    """

    endpoints: Union[Unset, "APIVersionSolrEndpoints"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        endpoints: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.endpoints, Unset):
            endpoints = self.endpoints.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if endpoints is not UNSET:
            field_dict["endpoints"] = endpoints

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.api_version_solr_endpoints import APIVersionSolrEndpoints

        d = src_dict.copy()
        _endpoints = d.pop("endpoints", UNSET)
        endpoints: Union[Unset, APIVersionSolrEndpoints]
        if isinstance(_endpoints, Unset):
            endpoints = UNSET
        else:
            endpoints = APIVersionSolrEndpoints.from_dict(_endpoints)

        api_version_solr = cls(
            endpoints=endpoints,
        )

        api_version_solr.additional_properties = d
        return api_version_solr

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.search_facet_bucket import SearchFacetBucket
    from ..models.search_facet_range_bucket import SearchFacetRangeBucket


T = TypeVar("T", bound="SearchFacet")


@_attrs_define
class SearchFacet:
    """An object containing search results for a facet

    Attributes:
        type (str): The type of facet
        num_buckets (int): The number of buckets in the facet
        buckets (Union[List['SearchFacetBucket'], List['SearchFacetRangeBucket']]):
        min_ (Union[Unset, Any]): TODO
        max_ (Union[Unset, Any]): TODO
        gap (Union[Unset, Any]): TODO
    """

    type: str
    num_buckets: int
    buckets: Union[List["SearchFacetBucket"], List["SearchFacetRangeBucket"]]
    min_: Union[Unset, Any] = UNSET
    max_: Union[Unset, Any] = UNSET
    gap: Union[Unset, Any] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        num_buckets = self.num_buckets

        buckets: List[Dict[str, Any]]
        if isinstance(self.buckets, list):
            buckets = []
            for buckets_type_0_item_data in self.buckets:
                buckets_type_0_item = buckets_type_0_item_data.to_dict()
                buckets.append(buckets_type_0_item)

        else:
            buckets = []
            for buckets_type_1_item_data in self.buckets:
                buckets_type_1_item = buckets_type_1_item_data.to_dict()
                buckets.append(buckets_type_1_item)

        min_ = self.min_

        max_ = self.max_

        gap = self.gap

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "type": type,
                "numBuckets": num_buckets,
                "buckets": buckets,
            }
        )
        if min_ is not UNSET:
            field_dict["min"] = min_
        if max_ is not UNSET:
            field_dict["max"] = max_
        if gap is not UNSET:
            field_dict["gap"] = gap

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.search_facet_bucket import SearchFacetBucket
        from ..models.search_facet_range_bucket import SearchFacetRangeBucket

        d = src_dict.copy()
        type = d.pop("type")

        num_buckets = d.pop("numBuckets")

        def _parse_buckets(data: object) -> Union[List["SearchFacetBucket"], List["SearchFacetRangeBucket"]]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                buckets_type_0 = []
                _buckets_type_0 = data
                for buckets_type_0_item_data in _buckets_type_0:
                    buckets_type_0_item = SearchFacetBucket.from_dict(buckets_type_0_item_data)

                    buckets_type_0.append(buckets_type_0_item)

                return buckets_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, list):
                raise TypeError()
            buckets_type_1 = []
            _buckets_type_1 = data
            for buckets_type_1_item_data in _buckets_type_1:
                buckets_type_1_item = SearchFacetRangeBucket.from_dict(buckets_type_1_item_data)

                buckets_type_1.append(buckets_type_1_item)

            return buckets_type_1

        buckets = _parse_buckets(d.pop("buckets"))

        min_ = d.pop("min", UNSET)

        max_ = d.pop("max", UNSET)

        gap = d.pop("gap", UNSET)

        search_facet = cls(
            type=type,
            num_buckets=num_buckets,
            buckets=buckets,
            min_=min_,
            max_=max_,
            gap=gap,
        )

        return search_facet



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.content_item_access_right import ContentItemAccessRight
from ..models.content_item_labels_item import ContentItemLabelsItem
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.collection import Collection
    from ..models.content_item_match import ContentItemMatch
    from ..models.content_item_mentions_item import ContentItemMentionsItem
    from ..models.content_item_region import ContentItemRegion
    from ..models.content_item_topic import ContentItemTopic
    from ..models.entity import Entity
    from ..models.newspaper import Newspaper
    from ..models.newspaper_issue import NewspaperIssue
    from ..models.page import Page


T = TypeVar("T", bound="ContentItem")


@_attrs_define
class ContentItem:
    """A journal/magazine content item (article, advertisement, etc.)

    Attributes:
        uid (str): The unique identifier of the content item
        type (str): The type of the content item. NOTE: may be empty.
        title (str): The title of the content item
        size (int): The size of the content item in characters
        nb_pages (int): The number of pages in this content item
        pages (List['Page']):
        is_cc (bool): TODO
        excerpt (str): The excerpt of the content item
        labels (List[ContentItemLabelsItem]): TODO
        access_right (ContentItemAccessRight):
        year (int): The year of the content item
        locations (Union[Unset, List['Entity']]):
        persons (Union[Unset, List['Entity']]):
        language (Union[Unset, str]): The language code of the content item
        issue (Union[Unset, NewspaperIssue]):
        matches (Union[Unset, List['ContentItemMatch']]):
        regions (Union[Unset, List['ContentItemRegion']]):
        region_breaks (Union[Unset, List[int]]):
        content_line_breaks (Union[Unset, List[int]]):
        is_front (Union[Unset, bool]): TODO
        date (Union[None, Unset, datetime.datetime]):
        country (Union[Unset, str]): The country code of the content item
        tags (Union[Unset, List[str]]):
        collections (Union[List['Collection'], List[str], Unset]):
        newspaper (Union[Unset, Newspaper]): A newspaper
        data_provider (Union[None, Unset, str]):
        topics (Union[Unset, List['ContentItemTopic']]):
        content (Union[Unset, str]): The content of the content item
        mentions (Union[Unset, List['ContentItemMentionsItem']]):
        v (Union[Unset, str]): TODO
    """

    uid: str
    type: str
    title: str
    size: int
    nb_pages: int
    pages: List["Page"]
    is_cc: bool
    excerpt: str
    labels: List[ContentItemLabelsItem]
    access_right: ContentItemAccessRight
    year: int
    locations: Union[Unset, List["Entity"]] = UNSET
    persons: Union[Unset, List["Entity"]] = UNSET
    language: Union[Unset, str] = UNSET
    issue: Union[Unset, "NewspaperIssue"] = UNSET
    matches: Union[Unset, List["ContentItemMatch"]] = UNSET
    regions: Union[Unset, List["ContentItemRegion"]] = UNSET
    region_breaks: Union[Unset, List[int]] = UNSET
    content_line_breaks: Union[Unset, List[int]] = UNSET
    is_front: Union[Unset, bool] = UNSET
    date: Union[None, Unset, datetime.datetime] = UNSET
    country: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    collections: Union[List["Collection"], List[str], Unset] = UNSET
    newspaper: Union[Unset, "Newspaper"] = UNSET
    data_provider: Union[None, Unset, str] = UNSET
    topics: Union[Unset, List["ContentItemTopic"]] = UNSET
    content: Union[Unset, str] = UNSET
    mentions: Union[Unset, List["ContentItemMentionsItem"]] = UNSET
    v: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        type = self.type

        title = self.title

        size = self.size

        nb_pages = self.nb_pages

        pages = []
        for pages_item_data in self.pages:
            pages_item = pages_item_data.to_dict()
            pages.append(pages_item)

        is_cc = self.is_cc

        excerpt = self.excerpt

        labels = []
        for labels_item_data in self.labels:
            labels_item = labels_item_data.value
            labels.append(labels_item)

        access_right = self.access_right.value

        year = self.year

        locations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.locations, Unset):
            locations = []
            for locations_item_data in self.locations:
                locations_item = locations_item_data.to_dict()
                locations.append(locations_item)

        persons: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.persons, Unset):
            persons = []
            for persons_item_data in self.persons:
                persons_item = persons_item_data.to_dict()
                persons.append(persons_item)

        language = self.language

        issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.issue, Unset):
            issue = self.issue.to_dict()

        matches: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.matches, Unset):
            matches = []
            for matches_item_data in self.matches:
                matches_item = matches_item_data.to_dict()
                matches.append(matches_item)

        regions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.regions, Unset):
            regions = []
            for regions_item_data in self.regions:
                regions_item = regions_item_data.to_dict()
                regions.append(regions_item)

        region_breaks: Union[Unset, List[int]] = UNSET
        if not isinstance(self.region_breaks, Unset):
            region_breaks = self.region_breaks

        content_line_breaks: Union[Unset, List[int]] = UNSET
        if not isinstance(self.content_line_breaks, Unset):
            content_line_breaks = self.content_line_breaks

        is_front = self.is_front

        date: Union[None, Unset, str]
        if isinstance(self.date, Unset):
            date = UNSET
        elif isinstance(self.date, datetime.datetime):
            date = self.date.isoformat()
        else:
            date = self.date

        country = self.country

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        collections: Union[List[Dict[str, Any]], List[str], Unset]
        if isinstance(self.collections, Unset):
            collections = UNSET
        elif isinstance(self.collections, list):
            collections = self.collections

        else:
            collections = []
            for collections_type_1_item_data in self.collections:
                collections_type_1_item = collections_type_1_item_data.to_dict()
                collections.append(collections_type_1_item)

        newspaper: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.newspaper, Unset):
            newspaper = self.newspaper.to_dict()

        data_provider: Union[None, Unset, str]
        if isinstance(self.data_provider, Unset):
            data_provider = UNSET
        else:
            data_provider = self.data_provider

        topics: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.topics, Unset):
            topics = []
            for topics_item_data in self.topics:
                topics_item = topics_item_data.to_dict()
                topics.append(topics_item)

        content = self.content

        mentions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.mentions, Unset):
            mentions = []
            for mentions_item_data in self.mentions:
                mentions_item = mentions_item_data.to_dict()
                mentions.append(mentions_item)

        v = self.v

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "type": type,
                "title": title,
                "size": size,
                "nbPages": nb_pages,
                "pages": pages,
                "isCC": is_cc,
                "excerpt": excerpt,
                "labels": labels,
                "accessRight": access_right,
                "year": year,
            }
        )
        if locations is not UNSET:
            field_dict["locations"] = locations
        if persons is not UNSET:
            field_dict["persons"] = persons
        if language is not UNSET:
            field_dict["language"] = language
        if issue is not UNSET:
            field_dict["issue"] = issue
        if matches is not UNSET:
            field_dict["matches"] = matches
        if regions is not UNSET:
            field_dict["regions"] = regions
        if region_breaks is not UNSET:
            field_dict["regionBreaks"] = region_breaks
        if content_line_breaks is not UNSET:
            field_dict["contentLineBreaks"] = content_line_breaks
        if is_front is not UNSET:
            field_dict["isFront"] = is_front
        if date is not UNSET:
            field_dict["date"] = date
        if country is not UNSET:
            field_dict["country"] = country
        if tags is not UNSET:
            field_dict["tags"] = tags
        if collections is not UNSET:
            field_dict["collections"] = collections
        if newspaper is not UNSET:
            field_dict["newspaper"] = newspaper
        if data_provider is not UNSET:
            field_dict["dataProvider"] = data_provider
        if topics is not UNSET:
            field_dict["topics"] = topics
        if content is not UNSET:
            field_dict["content"] = content
        if mentions is not UNSET:
            field_dict["mentions"] = mentions
        if v is not UNSET:
            field_dict["v"] = v

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.collection import Collection
        from ..models.content_item_match import ContentItemMatch
        from ..models.content_item_mentions_item import ContentItemMentionsItem
        from ..models.content_item_region import ContentItemRegion
        from ..models.content_item_topic import ContentItemTopic
        from ..models.entity import Entity
        from ..models.newspaper import Newspaper
        from ..models.newspaper_issue import NewspaperIssue
        from ..models.page import Page

        d = src_dict.copy()
        uid = d.pop("uid")

        type = d.pop("type")

        title = d.pop("title")

        size = d.pop("size")

        nb_pages = d.pop("nbPages")

        pages = []
        _pages = d.pop("pages")
        for pages_item_data in _pages:
            pages_item = Page.from_dict(pages_item_data)

            pages.append(pages_item)

        is_cc = d.pop("isCC")

        excerpt = d.pop("excerpt")

        labels = []
        _labels = d.pop("labels")
        for labels_item_data in _labels:
            labels_item = ContentItemLabelsItem(labels_item_data)

            labels.append(labels_item)

        access_right = ContentItemAccessRight(d.pop("accessRight"))

        year = d.pop("year")

        locations = []
        _locations = d.pop("locations", UNSET)
        for locations_item_data in _locations or []:
            locations_item = Entity.from_dict(locations_item_data)

            locations.append(locations_item)

        persons = []
        _persons = d.pop("persons", UNSET)
        for persons_item_data in _persons or []:
            persons_item = Entity.from_dict(persons_item_data)

            persons.append(persons_item)

        language = d.pop("language", UNSET)

        _issue = d.pop("issue", UNSET)
        issue: Union[Unset, NewspaperIssue]
        if isinstance(_issue, Unset):
            issue = UNSET
        else:
            issue = NewspaperIssue.from_dict(_issue)

        matches = []
        _matches = d.pop("matches", UNSET)
        for matches_item_data in _matches or []:
            matches_item = ContentItemMatch.from_dict(matches_item_data)

            matches.append(matches_item)

        regions = []
        _regions = d.pop("regions", UNSET)
        for regions_item_data in _regions or []:
            regions_item = ContentItemRegion.from_dict(regions_item_data)

            regions.append(regions_item)

        region_breaks = cast(List[int], d.pop("regionBreaks", UNSET))

        content_line_breaks = cast(List[int], d.pop("contentLineBreaks", UNSET))

        is_front = d.pop("isFront", UNSET)

        def _parse_date(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                date_type_0_type_0 = isoparse(data)

                return date_type_0_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        date = _parse_date(d.pop("date", UNSET))

        country = d.pop("country", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        def _parse_collections(data: object) -> Union[List["Collection"], List[str], Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                collections_type_0 = cast(List[str], data)

                return collections_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, list):
                raise TypeError()
            collections_type_1 = []
            _collections_type_1 = data
            for collections_type_1_item_data in _collections_type_1:
                collections_type_1_item = Collection.from_dict(collections_type_1_item_data)

                collections_type_1.append(collections_type_1_item)

            return collections_type_1

        collections = _parse_collections(d.pop("collections", UNSET))

        _newspaper = d.pop("newspaper", UNSET)
        newspaper: Union[Unset, Newspaper]
        if isinstance(_newspaper, Unset):
            newspaper = UNSET
        else:
            newspaper = Newspaper.from_dict(_newspaper)

        def _parse_data_provider(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        data_provider = _parse_data_provider(d.pop("dataProvider", UNSET))

        topics = []
        _topics = d.pop("topics", UNSET)
        for topics_item_data in _topics or []:
            topics_item = ContentItemTopic.from_dict(topics_item_data)

            topics.append(topics_item)

        content = d.pop("content", UNSET)

        mentions = []
        _mentions = d.pop("mentions", UNSET)
        for mentions_item_data in _mentions or []:
            mentions_item = ContentItemMentionsItem.from_dict(mentions_item_data)

            mentions.append(mentions_item)

        v = d.pop("v", UNSET)

        content_item = cls(
            uid=uid,
            type=type,
            title=title,
            size=size,
            nb_pages=nb_pages,
            pages=pages,
            is_cc=is_cc,
            excerpt=excerpt,
            labels=labels,
            access_right=access_right,
            year=year,
            locations=locations,
            persons=persons,
            language=language,
            issue=issue,
            matches=matches,
            regions=regions,
            region_breaks=region_breaks,
            content_line_breaks=content_line_breaks,
            is_front=is_front,
            date=date,
            country=country,
            tags=tags,
            collections=collections,
            newspaper=newspaper,
            data_provider=data_provider,
            topics=topics,
            content=content,
            mentions=mentions,
            v=v,
        )

        return content_item



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="FindTextReusePassagesAddons")


@_attrs_define
class FindTextReusePassagesAddons:
    """
    Attributes:
        newspaper (Union[Unset, Any]):
    """

    newspaper: Union[Unset, Any] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        newspaper = self.newspaper

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if newspaper is not UNSET:
            field_dict["newspaper"] = newspaper

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        newspaper = d.pop("newspaper", UNSET)

        find_text_reuse_passages_addons = cls(
            newspaper=newspaper,
        )

        return find_text_reuse_passages_addons



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.entity_details_type import EntityDetailsType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.location_specific_fields import LocationSpecificFields


T = TypeVar("T", bound="EntityDetails")


@_attrs_define
class EntityDetails:
    """An entity like location, person, etc

    Attributes:
        uid (str): Unique identifier of the entity
        name (str): Entity name
        type (EntityDetailsType):
        count_items (int): TODO
        count_mentions (int): Number of mentions of this entity in articles
        wikidata_id (Union[Unset, str]): ID of the entity in wikidata
        wikidata (Union[Unset, LocationSpecificFields]): Details of a wikidata entity
    """

    uid: str
    name: str
    type: EntityDetailsType
    count_items: int
    count_mentions: int
    wikidata_id: Union[Unset, str] = UNSET
    wikidata: Union[Unset, "LocationSpecificFields"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        name = self.name

        type = self.type.value

        count_items = self.count_items

        count_mentions = self.count_mentions

        wikidata_id = self.wikidata_id

        wikidata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.wikidata, Unset):
            wikidata = self.wikidata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "name": name,
                "type": type,
                "countItems": count_items,
                "countMentions": count_mentions,
            }
        )
        if wikidata_id is not UNSET:
            field_dict["wikidataId"] = wikidata_id
        if wikidata is not UNSET:
            field_dict["wikidata"] = wikidata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.location_specific_fields import LocationSpecificFields

        d = src_dict.copy()
        uid = d.pop("uid")

        name = d.pop("name")

        type = EntityDetailsType(d.pop("type"))

        count_items = d.pop("countItems")

        count_mentions = d.pop("countMentions")

        wikidata_id = d.pop("wikidataId", UNSET)

        _wikidata = d.pop("wikidata", UNSET)
        wikidata: Union[Unset, LocationSpecificFields]
        if isinstance(_wikidata, Unset):
            wikidata = UNSET
        else:
            wikidata = LocationSpecificFields.from_dict(_wikidata)

        entity_details = cls(
            uid=uid,
            name=name,
            type=type,
            count_items=count_items,
            count_mentions=count_mentions,
            wikidata_id=wikidata_id,
            wikidata=wikidata,
        )

        return entity_details



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.collection import Collection
    from ..models.entity import Entity
    from ..models.newspaper import Newspaper
    from ..models.topic import Topic
    from ..models.year import Year


T = TypeVar("T", bound="SearchFacetBucket")


@_attrs_define
class SearchFacetBucket:
    """Facet bucket

    Attributes:
        count (int): Number of items in the bucket
        val (str): Value of the 'type' element
        uid (Union[Unset, str]): UID of the 'type' element. Same as 'val'
        item (Union['Collection', 'Entity', 'Newspaper', 'Topic', 'Year', Unset]): The item in the bucket. Particular
            objct schema depends on the facet type
    """

    count: int
    val: str
    uid: Union[Unset, str] = UNSET
    item: Union["Collection", "Entity", "Newspaper", "Topic", "Year", Unset] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        from ..models.collection import Collection
        from ..models.entity import Entity
        from ..models.newspaper import Newspaper
        from ..models.topic import Topic

        count = self.count

        val = self.val

        uid = self.uid

        item: Union[Dict[str, Any], Unset]
        if isinstance(self.item, Unset):
            item = UNSET
        elif isinstance(self.item, Newspaper):
            item = self.item.to_dict()
        elif isinstance(self.item, Collection):
            item = self.item.to_dict()
        elif isinstance(self.item, Entity):
            item = self.item.to_dict()
        elif isinstance(self.item, Topic):
            item = self.item.to_dict()
        else:
            item = self.item.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "count": count,
                "val": val,
            }
        )
        if uid is not UNSET:
            field_dict["uid"] = uid
        if item is not UNSET:
            field_dict["item"] = item

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.collection import Collection
        from ..models.entity import Entity
        from ..models.newspaper import Newspaper
        from ..models.topic import Topic
        from ..models.year import Year

        d = src_dict.copy()
        count = d.pop("count")

        val = d.pop("val")

        uid = d.pop("uid", UNSET)

        def _parse_item(data: object) -> Union["Collection", "Entity", "Newspaper", "Topic", "Year", Unset]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                item_type_0 = Newspaper.from_dict(data)

                return item_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                item_type_1 = Collection.from_dict(data)

                return item_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                item_type_2 = Entity.from_dict(data)

                return item_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                item_type_3 = Topic.from_dict(data)

                return item_type_3
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            item_type_4 = Year.from_dict(data)

            return item_type_4

        item = _parse_item(d.pop("item", UNSET))

        search_facet_bucket = cls(
            count=count,
            val=val,
            uid=uid,
            item=item,
        )

        return search_facet_bucket



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.topic_related_topics_item import TopicRelatedTopicsItem
    from ..models.topic_word import TopicWord


T = TypeVar("T", bound="Topic")


@_attrs_define
class Topic:
    """A topic (TODO)

    Attributes:
        uid (str): The unique identifier of the topic
        language (str): The language code of the topic
        community (Union[Unset, str]): TODO
        pagerank (Union[Unset, float]): TODO
        degree (Union[Unset, float]): TODO
        x (Union[Unset, float]): TODO
        y (Union[Unset, float]): TODO
        related_topics (Union[Unset, List['TopicRelatedTopicsItem']]):
        count_items (Union[Unset, float]): TODO
        excerpt (Union[Unset, List['TopicWord']]): TODO
        words (Union[Unset, List['TopicWord']]): TODO
        model (Union[Unset, str]): ID of the model used to generate the topic
    """

    uid: str
    language: str
    community: Union[Unset, str] = UNSET
    pagerank: Union[Unset, float] = UNSET
    degree: Union[Unset, float] = UNSET
    x: Union[Unset, float] = UNSET
    y: Union[Unset, float] = UNSET
    related_topics: Union[Unset, List["TopicRelatedTopicsItem"]] = UNSET
    count_items: Union[Unset, float] = UNSET
    excerpt: Union[Unset, List["TopicWord"]] = UNSET
    words: Union[Unset, List["TopicWord"]] = UNSET
    model: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        language = self.language

        community = self.community

        pagerank = self.pagerank

        degree = self.degree

        x = self.x

        y = self.y

        related_topics: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.related_topics, Unset):
            related_topics = []
            for related_topics_item_data in self.related_topics:
                related_topics_item = related_topics_item_data.to_dict()
                related_topics.append(related_topics_item)

        count_items = self.count_items

        excerpt: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.excerpt, Unset):
            excerpt = []
            for excerpt_item_data in self.excerpt:
                excerpt_item = excerpt_item_data.to_dict()
                excerpt.append(excerpt_item)

        words: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.words, Unset):
            words = []
            for words_item_data in self.words:
                words_item = words_item_data.to_dict()
                words.append(words_item)

        model = self.model

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "language": language,
            }
        )
        if community is not UNSET:
            field_dict["community"] = community
        if pagerank is not UNSET:
            field_dict["pagerank"] = pagerank
        if degree is not UNSET:
            field_dict["degree"] = degree
        if x is not UNSET:
            field_dict["x"] = x
        if y is not UNSET:
            field_dict["y"] = y
        if related_topics is not UNSET:
            field_dict["relatedTopics"] = related_topics
        if count_items is not UNSET:
            field_dict["countItems"] = count_items
        if excerpt is not UNSET:
            field_dict["excerpt"] = excerpt
        if words is not UNSET:
            field_dict["words"] = words
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.topic_related_topics_item import TopicRelatedTopicsItem
        from ..models.topic_word import TopicWord

        d = src_dict.copy()
        uid = d.pop("uid")

        language = d.pop("language")

        community = d.pop("community", UNSET)

        pagerank = d.pop("pagerank", UNSET)

        degree = d.pop("degree", UNSET)

        x = d.pop("x", UNSET)

        y = d.pop("y", UNSET)

        related_topics = []
        _related_topics = d.pop("relatedTopics", UNSET)
        for related_topics_item_data in _related_topics or []:
            related_topics_item = TopicRelatedTopicsItem.from_dict(related_topics_item_data)

            related_topics.append(related_topics_item)

        count_items = d.pop("countItems", UNSET)

        excerpt = []
        _excerpt = d.pop("excerpt", UNSET)
        for excerpt_item_data in _excerpt or []:
            excerpt_item = TopicWord.from_dict(excerpt_item_data)

            excerpt.append(excerpt_item)

        words = []
        _words = d.pop("words", UNSET)
        for words_item_data in _words or []:
            words_item = TopicWord.from_dict(words_item_data)

            words.append(words_item)

        model = d.pop("model", UNSET)

        topic = cls(
            uid=uid,
            language=language,
            community=community,
            pagerank=pagerank,
            degree=degree,
            x=x,
            y=y,
            related_topics=related_topics,
            count_items=count_items,
            excerpt=excerpt,
            words=words,
            model=model,
        )

        return topic



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="APIVersionApiVersion")


@_attrs_define
class APIVersionApiVersion:
    """
    Attributes:
        branch (Union[Unset, str]):
        revision (Union[Unset, str]):
        version (Union[Unset, str]):
    """

    branch: Union[Unset, str] = UNSET
    revision: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        branch = self.branch

        revision = self.revision

        version = self.version

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if branch is not UNSET:
            field_dict["branch"] = branch
        if revision is not UNSET:
            field_dict["revision"] = revision
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        branch = d.pop("branch", UNSET)

        revision = d.pop("revision", UNSET)

        version = d.pop("version", UNSET)

        api_version_api_version = cls(
            branch=branch,
            revision=revision,
            version=version,
        )

        api_version_api_version.additional_properties = d
        return api_version_api_version

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.text_reuse_cluster_details_facets_item_buckets_item import (
        TextReuseClusterDetailsFacetsItemBucketsItem,
    )


T = TypeVar("T", bound="TextReuseClusterDetailsFacetsItem")


@_attrs_define
class TextReuseClusterDetailsFacetsItem:
    """
    Attributes:
        type (Union[Unset, str]): Facet type
        num_buckets (Union[Unset, int]): Number of buckets
        buckets (Union[Unset, List['TextReuseClusterDetailsFacetsItemBucketsItem']]):
    """

    type: Union[Unset, str] = UNSET
    num_buckets: Union[Unset, int] = UNSET
    buckets: Union[Unset, List["TextReuseClusterDetailsFacetsItemBucketsItem"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        num_buckets = self.num_buckets

        buckets: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.buckets, Unset):
            buckets = []
            for buckets_item_data in self.buckets:
                buckets_item = buckets_item_data.to_dict()
                buckets.append(buckets_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if num_buckets is not UNSET:
            field_dict["numBuckets"] = num_buckets
        if buckets is not UNSET:
            field_dict["buckets"] = buckets

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_cluster_details_facets_item_buckets_item import (
            TextReuseClusterDetailsFacetsItemBucketsItem,
        )

        d = src_dict.copy()
        type = d.pop("type", UNSET)

        num_buckets = d.pop("numBuckets", UNSET)

        buckets = []
        _buckets = d.pop("buckets", UNSET)
        for buckets_item_data in _buckets or []:
            buckets_item = TextReuseClusterDetailsFacetsItemBucketsItem.from_dict(buckets_item_data)

            buckets.append(buckets_item)

        text_reuse_cluster_details_facets_item = cls(
            type=type,
            num_buckets=num_buckets,
            buckets=buckets,
        )

        return text_reuse_cluster_details_facets_item



from pandas import DataFrame, json_normalize

from impresso.api_client.api.search_facets import get_tr_passages_facet
from impresso.api_client.api.text_reuse_passages import find_text_reuse_passages
from impresso.api_client.models.find_text_reuse_passages_order_by import (
    FindTextReusePassagesOrderBy,
    FindTextReusePassagesOrderByLiteral,
)
from impresso.api_client.models.get_tr_passages_facet_id import (
    GetTrPassagesFacetId,
    GetTrPassagesFacetIdLiteral,
)
from impresso.api_client.models.get_tr_passages_facet_order_by import (
    GetTrPassagesFacetOrderBy,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_models import BaseFind, SearchFacet, TextReusePassage
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.resources.search import FacetDataContainer
from impresso.resources.text_reuse.clusters import Range, _build_filters
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class FindTextReusePassageResponseSchema(BaseFind):
    """Schema for the text reuse passage response."""

    data: list[TextReusePassage]


class FindTextReusePassagesContainer(DataContainer):
    """Response of a find text reuse passages call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("id")
        return DataFrame()


class TextReusePassagesResource(Resource):
    """Text reuse passages resource."""

    name = "textReusePassages"

    def find(
        self,
        text: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        order_by: FindTextReusePassagesOrderByLiteral | None = None,
        cluster_id: str | AND[str] | OR[str] | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        title: str | AND[str] | OR[str] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        front_page: bool | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        language: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
    ) -> FindTextReusePassagesContainer:
        # reusing build filters from clusters - they are the same
        filters = _build_filters(
            cluster_id=cluster_id,
            cluster_size=cluster_size,
            title=title,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
            collection_id=collection_id,
            front_page=front_page,
            topic_id=topic_id,
            language=language,
            country=country,
            mention=mention,
            entity_id=entity_id,
        )
        if text is not None:
            filters.extend(and_or_filter(text, "string"))
        filters_pb = filters_as_protobuf(filters or [])

        result = find_text_reuse_passages.sync(
            client=self._api_client,
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindTextReusePassagesOrderBy)
                if order_by is not None
                else UNSET
            ),
            filters=filters_pb if filters_pb else UNSET,
        )
        raise_for_error(result)
        return FindTextReusePassagesContainer(
            result,
            FindTextReusePassageResponseSchema,
            web_app_search_result_url=_build_web_app_find_passages_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )

    def facet(
        self,
        facet: GetTrPassagesFacetIdLiteral,
        text: str | None = None,
        limit: int | None = None,
        offset: int | None = None,
        order_by: FindTextReusePassagesOrderByLiteral | None = None,
        cluster_id: str | AND[str] | OR[str] | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        title: str | AND[str] | OR[str] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        front_page: bool | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        language: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
    ) -> FacetDataContainer:
        facet_id = get_enum_from_literal(facet, GetTrPassagesFacetId)
        if isinstance(facet_id, Unset):
            raise ValueError(f"{facet} is not a valid value")

        filters = _build_filters(
            cluster_id=cluster_id,
            cluster_size=cluster_size,
            title=title,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
            collection_id=collection_id,
            front_page=front_page,
            topic_id=topic_id,
            language=language,
            country=country,
            mention=mention,
            entity_id=entity_id,
        )

        if text is not None:
            filters.extend(and_or_filter(text, "string"))

        filters_pb = filters_as_protobuf(filters or [])

        result = get_tr_passages_facet.sync(
            client=self._api_client,
            id=facet_id,
            filters=filters_pb if filters_pb else UNSET,
            offset=offset if offset is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, GetTrPassagesFacetOrderBy)
                if order_by is not None
                else get_enum_from_literal("value", GetTrPassagesFacetOrderBy)
            ),
        )
        raise_for_error(result)
        return FacetDataContainer(
            result,
            SearchFacet,
            limit=limit,
            offset=offset,
            web_app_search_result_url=_build_web_app_find_passages_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )


def _build_web_app_find_passages_url(
    base_url: str,
    filters=str | None,
    limit=int | None,
    offset=int | None,
    order_by=FindTextReusePassagesOrderBy | None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "sort": order_by,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/text-reuse/passages"
    return f"{url}?{query_string}" if query_string else url



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="ImpressoNamedEntityRecognitionRequest")


@_attrs_define
class ImpressoNamedEntityRecognitionRequest:
    """Request body for the Impresso NER endpoint

    Attributes:
        text (str): Text to be processed for named entity recognition
    """

    text: str

    def to_dict(self) -> Dict[str, Any]:
        text = self.text

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "text": text,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        text = d.pop("text")

        impresso_named_entity_recognition_request = cls(
            text=text,
        )

        return impresso_named_entity_recognition_request



import datetime
from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TextReuseClusterTimeCoverage")


@_attrs_define
class TextReuseClusterTimeCoverage:
    """Time window covered by documents in the cluster

    Attributes:
        from_ (Union[Unset, datetime.date]):
        to (Union[Unset, datetime.date]):
    """

    from_: Union[Unset, datetime.date] = UNSET
    to: Union[Unset, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        from_: Union[Unset, str] = UNSET
        if not isinstance(self.from_, Unset):
            from_ = self.from_.isoformat()

        to: Union[Unset, str] = UNSET
        if not isinstance(self.to, Unset):
            to = self.to.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _from_ = d.pop("from", UNSET)
        from_: Union[Unset, datetime.date]
        if isinstance(_from_, Unset):
            from_ = UNSET
        else:
            from_ = isoparse(_from_).date()

        _to = d.pop("to", UNSET)
        to: Union[Unset, datetime.date]
        if isinstance(_to, Unset):
            to = UNSET
        else:
            to = isoparse(_to).date()

        text_reuse_cluster_time_coverage = cls(
            from_=from_,
            to=to,
        )

        return text_reuse_cluster_time_coverage



from typing import Literal

from pandas import DataFrame, json_normalize

from impresso.api_client.api.entities import find_entities, get_entity
from impresso.api_client.models.find_entities_order_by import (
    FindEntitiesOrderBy,
    FindEntitiesOrderByLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind, EntityDetails, Filter
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.structures import AND, OR
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class FindEntitiesSchema(BaseFind):
    """Schema for the find entities response."""

    data: list[EntityDetails]


class FindEntitiesContainer(DataContainer):
    """Response of a find call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()


class GetEntityContainer(DataContainer):
    """Response of a get call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()
        if len(data):
            return json_normalize([self._data.to_dict()]).set_index("uid")
        return DataFrame()


EntityType = Literal["person", "location"]


class EntitiesResource(Resource):
    """Work with entities"""

    name = "entities"

    def find(
        self,
        q: str | None = None,
        wikidata_id: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
        entity_type: EntityType | AND[EntityType] | OR[EntityType] | None = None,
        order_by: FindEntitiesOrderByLiteral | None = None,
        load_wikidata: bool = False,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindEntitiesContainer:
        """Find entities."""

        filters: list[Filter] = []
        if entity_type is not None:
            filters.extend(and_or_filter(entity_type, "type"))  # type: ignore
        if wikidata_id is not None:
            filters.extend(and_or_filter(wikidata_id, "wikidata_id"))
        if entity_id is not None:
            filters.extend(and_or_filter(entity_id, "uid"))

        filters_pb = filters_as_protobuf(filters or [])

        result = find_entities.sync(
            client=self._api_client,
            q=q if q is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindEntitiesOrderBy)
                if order_by is not None
                else UNSET
            ),
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            filters=filters_pb if filters_pb else UNSET,
            resolve=load_wikidata,
        )
        raise_for_error(result)
        return FindEntitiesContainer(
            result,
            FindEntitiesSchema,
            web_app_search_result_url=(
                _build_web_app_find_entities_url(
                    base_url=self._get_web_app_base_url(),
                    q=q,
                )
                if wikidata_id is None and entity_type is None
                else None
            ),
        )

    def get(self, id: str) -> GetEntityContainer:
        """Get entity by ID."""

        result = get_entity.sync(
            client=self._api_client,
            id=id,
        )
        raise_for_error(result)
        return GetEntityContainer(
            result,
            FindEntitiesSchema,
            web_app_search_result_url=_build_web_app_get_entity_url(
                base_url=self._get_web_app_base_url(),
                id=id,
            ),
        )


def _build_web_app_find_entities_url(
    base_url: str,
    q: str | None = None,
) -> str:
    query_params = {
        "q": q,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/entities"
    return f"{url}?{query_string}" if query_string else url


def _build_web_app_get_entity_url(
    base_url: str,
    id: str,
) -> str:
    return f"{base_url}/entities/{id}"



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.year_weights import YearWeights


T = TypeVar("T", bound="Year")


@_attrs_define
class Year:
    """A year (TODO)

    Attributes:
        uid (Union[Unset, int]): Numeric representation of the year
        values (Union[Unset, YearWeights]): Total items counts within a year
        refs (Union[Unset, YearWeights]): Total items counts within a year
    """

    uid: Union[Unset, int] = UNSET
    values: Union[Unset, "YearWeights"] = UNSET
    refs: Union[Unset, "YearWeights"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        values: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.values, Unset):
            values = self.values.to_dict()

        refs: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.refs, Unset):
            refs = self.refs.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if uid is not UNSET:
            field_dict["uid"] = uid
        if values is not UNSET:
            field_dict["values"] = values
        if refs is not UNSET:
            field_dict["refs"] = refs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.year_weights import YearWeights

        d = src_dict.copy()
        uid = d.pop("uid", UNSET)

        _values = d.pop("values", UNSET)
        values: Union[Unset, YearWeights]
        if isinstance(_values, Unset):
            values = UNSET
        else:
            values = YearWeights.from_dict(_values)

        _refs = d.pop("refs", UNSET)
        refs: Union[Unset, YearWeights]
        if isinstance(_refs, Unset):
            refs = UNSET
        else:
            refs = YearWeights.from_dict(_refs)

        year = cls(
            uid=uid,
            values=values,
            refs=refs,
        )

        return year



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchFacetRangeBucket")


@_attrs_define
class SearchFacetRangeBucket:
    """Facet bucket

    Attributes:
        count (int): Number of items in the bucket
        val (int): Value of the 'type' element
        lower (Union[Unset, int]): Lower bound of the range
        upper (Union[Unset, int]): Lower bound of the range
    """

    count: int
    val: int
    lower: Union[Unset, int] = UNSET
    upper: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        count = self.count

        val = self.val

        lower = self.lower

        upper = self.upper

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "count": count,
                "val": val,
            }
        )
        if lower is not UNSET:
            field_dict["lower"] = lower
        if upper is not UNSET:
            field_dict["upper"] = upper

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        count = d.pop("count")

        val = d.pop("val")

        lower = d.pop("lower", UNSET)

        upper = d.pop("upper", UNSET)

        search_facet_range_bucket = cls(
            count=count,
            val=val,
            lower=lower,
            upper=upper,
        )

        return search_facet_range_bucket



from enum import Enum
from typing import Literal


class TextReuseClusterDetailsResolution(str, Enum):
    DAY = "day"
    MONTH = "month"
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)


TextReuseClusterDetailsResolutionLiteral = Literal[
    "day",
    "month",
    "year",
]



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="NewspaperIssue")


@_attrs_define
class NewspaperIssue:
    """
    Attributes:
        uid (str): The unique identifier of the issue
        cover (str): TODO
        labels (List[str]): The labels of the issue
        fresh (bool): TODO
        access_rights (str): TODO: list available options
        date (Union[Unset, datetime.datetime]): The date of the issue
        year (Union[Unset, str]): The year of the issue
    """

    uid: str
    cover: str
    labels: List[str]
    fresh: bool
    access_rights: str
    date: Union[Unset, datetime.datetime] = UNSET
    year: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        cover = self.cover

        labels = self.labels

        fresh = self.fresh

        access_rights = self.access_rights

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        year = self.year

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "cover": cover,
                "labels": labels,
                "fresh": fresh,
                "accessRights": access_rights,
            }
        )
        if date is not UNSET:
            field_dict["date"] = date
        if year is not UNSET:
            field_dict["year"] = year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        cover = d.pop("cover")

        labels = cast(List[str], d.pop("labels"))

        fresh = d.pop("fresh")

        access_rights = d.pop("accessRights")

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.datetime]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date)

        year = d.pop("year", UNSET)

        newspaper_issue = cls(
            uid=uid,
            cover=cover,
            labels=labels,
            fresh=fresh,
            access_rights=access_rights,
            date=date,
            year=year,
        )

        return newspaper_issue



from enum import Enum
from typing import Literal


class FindNewspapersOrderBy(str, Enum):
    COUNTISSUES = "countIssues"
    ENDYEAR = "endYear"
    FIRSTISSUE = "firstIssue"
    LASTISSUE = "lastIssue"
    NAME = "name"
    STARTYEAR = "startYear"
    VALUE_0 = "-name"
    VALUE_11 = "-countIssues"
    VALUE_2 = "-startYear"
    VALUE_4 = "-endYear"
    VALUE_7 = "-firstIssue"
    VALUE_9 = "-lastIssue"

    def __str__(self) -> str:
        return str(self.value)


FindNewspapersOrderByLiteral = Literal[
    "countIssues",
    "endYear",
    "firstIssue",
    "lastIssue",
    "name",
    "startYear",
    "-name",
    "-countIssues",
    "-startYear",
    "-endYear",
    "-firstIssue",
    "-lastIssue",
]



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.text_reuse_cluster_compound import TextReuseClusterCompound


T = TypeVar("T", bound="FindTextReuseClustersResponse")


@_attrs_define
class FindTextReuseClustersResponse:
    """Response for GET /text-reuse-clusters

    Attributes:
        clusters (List['TextReuseClusterCompound']):
        info (Any):
    """

    clusters: List["TextReuseClusterCompound"]
    info: Any

    def to_dict(self) -> Dict[str, Any]:
        clusters = []
        for clusters_item_data in self.clusters:
            clusters_item = clusters_item_data.to_dict()
            clusters.append(clusters_item)

        info = self.info

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "clusters": clusters,
                "info": info,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_cluster_compound import TextReuseClusterCompound

        d = src_dict.copy()
        clusters = []
        _clusters = d.pop("clusters")
        for clusters_item_data in _clusters:
            clusters_item = TextReuseClusterCompound.from_dict(clusters_item_data)

            clusters.append(clusters_item)

        info = d.pop("info")

        find_text_reuse_clusters_response = cls(
            clusters=clusters,
            info=info,
        )

        return find_text_reuse_clusters_response



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.collectable_item_group_content_type import CollectableItemGroupContentType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.collection import Collection


T = TypeVar("T", bound="CollectableItemGroup")


@_attrs_define
class CollectableItemGroup:
    """Collectable item group object

    Attributes:
        item_id (Union[Unset, str]): The id of the collectable item group
        content_type (Union[Unset, CollectableItemGroupContentType]): Content type of the collectable item group:
            (A)rticle, (E)ntities, (P)ages, (I)ssues
        collection_ids (Union[Unset, List[str]]): Ids of the collections
        search_queries (Union[Unset, List[str]]): Search queries
        collections (Union[Unset, List['Collection']]): Collection objects
        latest_date_added (Union[Unset, datetime.datetime]): The latest date added to the collectable item group
    """

    item_id: Union[Unset, str] = UNSET
    content_type: Union[Unset, CollectableItemGroupContentType] = UNSET
    collection_ids: Union[Unset, List[str]] = UNSET
    search_queries: Union[Unset, List[str]] = UNSET
    collections: Union[Unset, List["Collection"]] = UNSET
    latest_date_added: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        item_id = self.item_id

        content_type: Union[Unset, str] = UNSET
        if not isinstance(self.content_type, Unset):
            content_type = self.content_type.value

        collection_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.collection_ids, Unset):
            collection_ids = self.collection_ids

        search_queries: Union[Unset, List[str]] = UNSET
        if not isinstance(self.search_queries, Unset):
            search_queries = self.search_queries

        collections: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.collections, Unset):
            collections = []
            for collections_item_data in self.collections:
                collections_item = collections_item_data.to_dict()
                collections.append(collections_item)

        latest_date_added: Union[Unset, str] = UNSET
        if not isinstance(self.latest_date_added, Unset):
            latest_date_added = self.latest_date_added.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if item_id is not UNSET:
            field_dict["itemId"] = item_id
        if content_type is not UNSET:
            field_dict["contentType"] = content_type
        if collection_ids is not UNSET:
            field_dict["collectionIds"] = collection_ids
        if search_queries is not UNSET:
            field_dict["searchQueries"] = search_queries
        if collections is not UNSET:
            field_dict["collections"] = collections
        if latest_date_added is not UNSET:
            field_dict["latestDateAdded"] = latest_date_added

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.collection import Collection

        d = src_dict.copy()
        item_id = d.pop("itemId", UNSET)

        _content_type = d.pop("contentType", UNSET)
        content_type: Union[Unset, CollectableItemGroupContentType]
        if isinstance(_content_type, Unset):
            content_type = UNSET
        else:
            content_type = CollectableItemGroupContentType(_content_type)

        collection_ids = cast(List[str], d.pop("collectionIds", UNSET))

        search_queries = cast(List[str], d.pop("searchQueries", UNSET))

        collections = []
        _collections = d.pop("collections", UNSET)
        for collections_item_data in _collections or []:
            collections_item = Collection.from_dict(collections_item_data)

            collections.append(collections_item)

        _latest_date_added = d.pop("latestDateAdded", UNSET)
        latest_date_added: Union[Unset, datetime.datetime]
        if isinstance(_latest_date_added, Unset):
            latest_date_added = UNSET
        else:
            latest_date_added = isoparse(_latest_date_added)

        collectable_item_group = cls(
            item_id=item_id,
            content_type=content_type,
            collection_ids=collection_ids,
            search_queries=search_queries,
            collections=collections,
            latest_date_added=latest_date_added,
        )

        collectable_item_group.additional_properties = d
        return collectable_item_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.content_item import ContentItem
    from ..models.find_content_item_response_200_info import FindContentItemResponse200Info


T = TypeVar("T", bound="FindContentItemResponse200")


@_attrs_define
class FindContentItemResponse200:
    """
    Attributes:
        limit (int): The number of items returned in this response
        offset (int): Starting index of the items subset returned in this response
        total (int): The total number of items matching the query
        info (FindContentItemResponse200Info): Additional information about the response.
        data (List['ContentItem']):
    """

    limit: int
    offset: int
    total: int
    info: "FindContentItemResponse200Info"
    data: List["ContentItem"]

    def to_dict(self) -> Dict[str, Any]:
        limit = self.limit

        offset = self.offset

        total = self.total

        info = self.info.to_dict()

        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "limit": limit,
                "offset": offset,
                "total": total,
                "info": info,
                "data": data,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.content_item import ContentItem
        from ..models.find_content_item_response_200_info import FindContentItemResponse200Info

        d = src_dict.copy()
        limit = d.pop("limit")

        offset = d.pop("offset")

        total = d.pop("total")

        info = FindContentItemResponse200Info.from_dict(d.pop("info"))

        data = []
        _data = d.pop("data")
        for data_item_data in _data:
            data_item = ContentItem.from_dict(data_item_data)

            data.append(data_item)

        find_content_item_response_200 = cls(
            limit=limit,
            offset=offset,
            total=total,
            info=info,
            data=data,
        )

        return find_content_item_response_200
